/* 
	Copyright (C) 2004 Alex de Landgraaf <alextreme@xs4all.nl>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <parted/parted.h>

#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "pm_libparted.h"
#include "pm_fswrap.h"
#include "pm_wrap.h"
#include "ui.h"
#include "wizard.h"

/*
  Note: read the parted-doc API first!
  this can be found in /usr/share/doc/parted-doc
  You'll need it to understand certain sections, and pm_lib

  Some guidelines for the source:

  when everything is set up using scan_partitions and scan_devices in 
  pm_lib, the partitions- and devices arrays are filled with the current 
  partitiontable information

  The current selected device can be retrieved with devices[selected_device],
  The current selected partition can be retrieved with partitions[selected_partition]
  
  An Extended partition contains Logical partitions or free space.

  Do use initialise_partinfo to clean out the partinfo array
*/

extern  GtkWidget *MainWindow;
extern  GtkWidget *CreateWindow;
extern  GtkWidget *ResizeWindow;
extern  GtkWidget *ApplyWindow;
extern  GtkWidget *RevertWindow;
extern  GtkWidget *DialogWindow;
extern  GtkWidget *HelpWindow;
extern  GtkWidget *DeleteWindow;
extern  GtkWidget *ConvertWindow;
extern  GtkWidget *MoveWindow;
extern  GtkWidget *TableWindow;
extern  GtkWidget *ErrorWindow;
extern  GtkWidget *ProgressWindow;
extern  GtkWidget *WizardWindow;

enum {
	PART_NUMBER,
	PART_PATH,
	PART_TYPE,
	PART_FS,
	PART_SIZE,
	PART_START,
	PART_END,
	PART_FLAGS,
	N_COLUMNS
};

void
on_ErrorOkButton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(ErrorWindow);
}

void
on_CreateButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar buffer[64];
	// GtkObject *adjustment;
	GtkWidget *size_label = lookup_widget(CreateWindow,"CreateMaxLabel");
	GtkWidget *size_entry = lookup_widget(CreateWindow,"CreateSizeEntry");

	read_fs_items(); // need this to set all available filesystems
	set_create_fs_option_menu(); // set up the option menu

	sprintf(buffer,"%ld",partinfo[selected_partition].size);
	gtk_entry_set_text(GTK_ENTRY(size_entry),buffer);
	sprintf(buffer,"%ld MB",partinfo[selected_partition].size);
	gtk_label_set_text(GTK_LABEL(size_label),buffer);
/*	adjustment = gtk_adjustment_new((gdouble)partinfo[selected_partition].size,
					 (gdouble)0,
					 (gdouble)partinfo[selected_partition].size,
					 1.0,
					 100.0,
					 (gdouble)0);
	gtk_range_set_adjustment(GTK_RANGE(range),GTK_ADJUSTMENT(adjustment));*/
	gtk_widget_show (CreateWindow);
	untoggle_create_types();
}


void
on_ResizeButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar buffer[64];
	GtkObject *adjustment;
	GtkWidget *device_label = lookup_widget(ResizeWindow,"ResizeDeviceLabel");
	GtkWidget *fs_label = lookup_widget(ResizeWindow,"ResizeFsLabel");
	GtkWidget *max_size_label = lookup_widget(ResizeWindow,"ResizeMaxLabel");
	GtkWidget *min_size_label = lookup_widget(ResizeWindow,"ResizeMinLabel");
	GtkWidget *size_label = lookup_widget(ResizeWindow,"ResizeSizeLabel");
	gtk_label_set_text(GTK_LABEL(device_label),partinfo[selected_partition].partname);
	gtk_label_set_text(GTK_LABEL(fs_label),partinfo[selected_partition].temp_fs_name);
	sprintf(buffer,"%ld MB",partinfo[selected_partition].min_size);
	gtk_label_set_text(GTK_LABEL(min_size_label),buffer);
	sprintf(buffer,"%ld MB",partinfo[selected_partition].max_size);
	gtk_label_set_text(GTK_LABEL(max_size_label),buffer);
	sprintf(buffer,"%ld MB",partinfo[selected_partition].size);
	gtk_label_set_text(GTK_LABEL(size_label),buffer);
	gtk_widget_show (ResizeWindow);
}


void
on_DeleteButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar buffer[64];
	GtkWidget *path_label = lookup_widget(DeleteWindow,"DeletePathLabel");
	GtkWidget *fs_label = lookup_widget(DeleteWindow,"DeleteFsLabel");
	GtkWidget *size_label = lookup_widget(DeleteWindow,"DeleteSizeLabel");
	gtk_label_set_text(GTK_LABEL(path_label),partinfo[selected_partition].partname);
	if (strcmp(partinfo[selected_partition].temp_fs_name,"") != 0) {
		gtk_widget_show(fs_label);
		gtk_label_set_text(GTK_LABEL(fs_label),partinfo[selected_partition].temp_fs_name);
	}
	else {
		gtk_widget_hide(fs_label);
	}
	sprintf(buffer,"%ld MB",partinfo[selected_partition].size);
	gtk_label_set_text(GTK_LABEL(size_label),buffer);

	gtk_widget_show (DeleteWindow);
}


void
on_ConvertButton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar buffer[64];
	GtkWidget *path_label = lookup_widget(ConvertWindow,"ConvertPathLabel");
	GtkWidget *fs_label = lookup_widget(ConvertWindow,"ConvertFsLabel");
	GtkWidget *size_label = lookup_widget(ConvertWindow,"ConvertSizeLabel");
	gtk_label_set_text(GTK_LABEL(path_label),partinfo[selected_partition].partname);
	gtk_label_set_text(GTK_LABEL(fs_label),partinfo[selected_partition].temp_fs_name);
	sprintf(buffer,"%ld MB",partinfo[selected_partition].size);
	gtk_label_set_text(GTK_LABEL(size_label),buffer);
	gtk_widget_show (ConvertWindow);
}


void
on_RefreshButton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	refresh_devices_and_partitions();
}


void
on_ApplyButton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_RevertButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_QuitButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	//ped_device_close(devices[selected_device]);
	gtk_main_quit();
	//gtk_widget_show (DialogWindow);
}


void
on_HelpButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_show(HelpWindow);
}

void
on_treeview1_row_activated             (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{

}

void
on_begin_of_free_space1_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_end_of_free_space1_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_MainHarddiskOptionMenu_clicked      (GtkOptionMenu   *optionmenu,
                                        gpointer         user_data)
{

}


void
on_MainWindow_show                     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *tree = lookup_widget(MainWindow,"treeview1");
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *item;
	gint i;

	GtkWidget *option = lookup_widget(widget,"MainHarddiskOptionMenu");

	pm_debug("Debug: devices: %d\n",device_count);

	for (i = 0; i < device_count;i++) {
		item = gtk_menu_item_new_with_label(devices[i]->path);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu),item);
		gtk_widget_show(item);
	}
	gtk_option_menu_set_menu(GTK_OPTION_MENU(option),menu);
	pm_get_filesystem_list();
	pm_select_device(0);
	set_callback_partition_list();
	//ped_device_open(devices[selected_device]);

	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	/* (partition path, /dev/hda1) */

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
		_("Path"),
		renderer,
		"text",PART_PATH,
		NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree),column);

	/* (partition type) */

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
		_("Type"),
		renderer,
		"text",PART_TYPE,
		NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree),column);

/* (fs type) */

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
		_("Filesystem"),
		renderer,
		"text",PART_FS,
		NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree),column);
	/* (partition size in MB) */

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
		_("Size (MB)"),
		renderer,
		"text",PART_SIZE,
		NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree),column);

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
		_("Flags"),
		renderer,
		"text",PART_FLAGS,
		NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree),column);

	gdk_notify_startup_complete ();
}

void
on_MainHarddiskOptionMenu_changed      (GtkOptionMenu   *optionmenu,
                                        gpointer         user_data)
{
	pm_select_update_device(gtk_option_menu_get_history(optionmenu));
	
	update_interface(selected_device);
	if (update_partition_list() == FALSE) {
		gtk_widget_show(TableWindow);
	}
	else {
		update_partition_widget();
		click_part_item(NULL,&partinfo[1]);
	}
}


void
on_MoveButton_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar buffer[64];
	GtkWidget *path_label = lookup_widget(MoveWindow,"MovePathLabel");
	GtkWidget *fs_label = lookup_widget(MoveWindow,"MoveFsLabel");
	GtkWidget *size_label = lookup_widget(MoveWindow,"MoveSizeLabel");
	gtk_label_set_text(GTK_LABEL(path_label),partinfo[selected_partition].partname);
	gtk_label_set_text(GTK_LABEL(fs_label),partinfo[selected_partition].temp_fs_name);
	sprintf(buffer,"%ld MB",partinfo[selected_partition].size);
	gtk_label_set_text(GTK_LABEL(size_label),buffer);
	gtk_widget_show(MoveWindow);
}

void
on_cancelbutton7_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_okbutton8_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_OkMoveButton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	pm_debug("Debug: Move not yet implemented!\n");
/*
	do_move(&devices[selected_device],partinfo[selected_partition].nr,partinfo[selected_partition].partname,(PedSector) start, (PedSector) end);
*/
	gtk_widget_hide(MoveWindow);
	refresh_devices_and_partitions();
}


void
on_CreateOkButton_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	show_busy_cursor( ((GtkWidget*)button)->window );
	long long mb_size = 0;
	long long sector_num = 0;
	gboolean success;
	gchar fs_buffer[64];
	PedSector end_sector;
	PedPartitionType part_type;
	GtkWidget *size = lookup_widget(CreateWindow,"CreateSizeEntry");
	GtkWidget *optionmenu_fs = lookup_widget(CreateWindow,"optionmenu4");
	GtkWidget *radio_primary_type = lookup_widget(CreateWindow,"CreatePrimaryRadiobutton");
	GtkWidget *radio_extended_type = lookup_widget(CreateWindow,"CreateExtendedRadiobutton");
	GtkWidget *radio_logical_type = lookup_widget(CreateWindow,"CreateLogicalRadiobutton");
	GtkWidget *check_boot_flag = lookup_widget(CreateWindow,"CreateBootFlagCheckbutton");
	GtkWidget *check_root_flag = lookup_widget(CreateWindow,"CreateRootFlagCheckbutton");
	GtkWidget *check_hidden_flag = lookup_widget(CreateWindow,"CreateHiddenFlagCheckbutton");
	GtkWidget *check_raid_flag = lookup_widget(CreateWindow,"CreateRaidFlagCheckbutton");
	GtkWidget *check_lvm_flag = lookup_widget(CreateWindow,"CreateLVMFlagCheckbutton");
	GtkWidget *check_lba_flag = lookup_widget(CreateWindow,"CreateLBAFlagCheckbutton");

	int saved_fd, stderr_pipe;
	gchar error_message_buffer[1000];
	gchar *value = NULL;
	int index;

	gint sel = gtk_option_menu_get_history(GTK_OPTION_MENU(optionmenu_fs));
	
	strcpy(fs_buffer,fs_menu_items[sel]);

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_primary_type)))
		part_type = 0;
	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_extended_type)))
		part_type = PED_PARTITION_EXTENDED;
	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_logical_type)))
		part_type = PED_PARTITION_LOGICAL;
	else {
		pm_debug("Error: invalid partition type\n");
		return;
	}
	value = gtk_entry_get_text(GTK_ENTRY(size));
	mb_size = (long long)atol(value);
	sector_num = pm_mb_to_pedsector(mb_size);
	if (mb_size <= 0 || sector_num <= 0 || mb_size > partinfo[selected_partition].size) {
		pm_debug("Debug: entered size is invalid, aborting\n");
		return;
	}

	end_sector = sector_num + partinfo[selected_partition].start;
	pm_debug("Debug: selectedpartition: %d\n",selected_partition);
	pm_debug("Debug: mbsize: %lld, start: %lld end: %lld\n",mb_size, partinfo[selected_partition].start,end_sector);

	redirect_stderr_to_pipe(&saved_fd, &stderr_pipe);

	if ( part_type == PED_PARTITION_EXTENDED || sel == 0 ) {
		success=do_mkpart(&devices[selected_device],
			part_type, /* Partition type == PED_PARTITION_EXTENDED */
			partinfo[selected_partition].start,
			(PedSector)end_sector);
	} else {
		success=do_mkpartfs(&devices[selected_device],
			part_type, /* Partition type == PED_PARTITION_* */
		        fs_buffer,
			selected_partition,
			partinfo[selected_partition].start,
			(PedSector)end_sector);
	}

	if ( success ) {
		unredirect_stderr(saved_fd, stderr_pipe);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_boot_flag)))
			pm_set_part_flag_state(&devices[selected_device],partinfo[selected_partition].num,PED_PARTITION_BOOT,1);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_root_flag)))
			pm_set_part_flag_state(&devices[selected_device],partinfo[selected_partition].num,PED_PARTITION_ROOT,1);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_hidden_flag)))
			pm_set_part_flag_state(&devices[selected_device],partinfo[selected_partition].num,PED_PARTITION_HIDDEN,1);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_raid_flag)))
			pm_set_part_flag_state(&devices[selected_device],partinfo[selected_partition].num,PED_PARTITION_RAID,1);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_lvm_flag)))
			pm_set_part_flag_state(&devices[selected_device],partinfo[selected_partition].num,PED_PARTITION_LVM,1);
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_lba_flag)))
			pm_set_part_flag_state(&devices[selected_device],partinfo[selected_partition].num,PED_PARTITION_LBA,1);
	} else {
		index = read(stderr_pipe, error_message_buffer, 999);
		error_message_buffer[index] = '\0';
		unredirect_stderr(saved_fd, stderr_pipe);
		show_error_message(error_message_buffer);
	}

	gtk_widget_hide(CreateWindow);	 
	show_normal_cursor( ((GtkWidget*)button)->window );
	refresh_devices_and_partitions();  
}

void
on_ResizeOkButton_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	int saved_fd, stderr_pipe;
	gchar error_message_buffer[1000];
	int index;

	PedSector end_sector;
	GtkWidget *entry = lookup_widget(ResizeWindow,"entry2");
	
	long long mb_size = (long long)atol(gtk_entry_get_text(GTK_ENTRY(entry)));
	long long sector_num = pm_mb_to_pedsector(mb_size);
	
	pm_debug("Debug: %lld : %Ld\n",sector_num,sector_num);
	end_sector = sector_num + partinfo[selected_partition].start;
	pm_debug("Debug: mbsize: %lld, end: %lld\n",mb_size,end_sector);
	/* check if success? */
	redirect_stderr_to_pipe(&saved_fd, &stderr_pipe);
	gboolean success = do_resize(&devices[selected_device],partinfo[selected_partition].num,partinfo[selected_partition].partname,(PedSector) partinfo[selected_partition].start,(PedSector) end_sector);
	if (success) {
		unredirect_stderr(saved_fd, stderr_pipe);
	}
	else {
		index = read(stderr_pipe, error_message_buffer, 999);
		error_message_buffer[index] = '\0';
		unredirect_stderr(saved_fd, stderr_pipe);
		show_error_message(error_message_buffer);
	}
	gtk_widget_hide(ResizeWindow);
	show_normal_cursor( ((GtkWidget*)button)->window );
	refresh_devices_and_partitions();
}


void
on_DeleteOkButton_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	int saved_fd, stderr_pipe;
	gchar error_message_buffer[1000];
	int index;

	redirect_stderr_to_pipe(&saved_fd, &stderr_pipe);
	show_busy_cursor( ((GtkWidget*)button)->window );
	gboolean success = do_rm(&devices[selected_device],partinfo[selected_partition].num);
	if (success) {
		unredirect_stderr(saved_fd, stderr_pipe);
	}
	else {
		index = read(stderr_pipe, error_message_buffer, 999);
		error_message_buffer[index] = '\0';
		unredirect_stderr(saved_fd, stderr_pipe);
		show_error_message(error_message_buffer);
	}
	gtk_widget_hide(DeleteWindow);
	show_normal_cursor( ((GtkWidget*)button)->window );
	refresh_devices_and_partitions();
}


void
on_ConvertOkButton_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	pm_debug("Debug: Convert not yet implemented\n");
	gtk_widget_hide(ConvertWindow);
	refresh_devices_and_partitions();
}


void
on_primary1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_TableWindow_show                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *device_label = lookup_widget(TableWindow,"TableDeviceLabel");
	gtk_label_set_text(GTK_LABEL(device_label),devices[selected_device]->path);
	
}

void
on_TableOkButton_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *option_menu = lookup_widget(TableWindow,"optionmenu7");
	PedDiskType *type = NULL;
	gint sel = gtk_option_menu_get_history(GTK_OPTION_MENU(option_menu));
	switch (sel) {
	case (0):
		type = ped_disk_type_get("msdos");
		break;
	case (1):
		type = ped_disk_type_get("bsd");
		break;
	case (2):
		type = ped_disk_type_get("gpt");
		break;
	case (3):
		type = ped_disk_type_get("mac");
		break;
	case (4):
		type = ped_disk_type_get("mips");
		break;
	case (5):
		type = ped_disk_type_get("pc98");
		break;
	case (6):
		type = ped_disk_type_get("sun");
		break;
	case (7):
		type = ped_disk_type_get("loop");
		break;
	default:
		pm_debug("Error: type not known!\n");
		return;
	}
	if (type == NULL) {
		pm_debug("Error: type not known!\n");
		return;
	}
	do_mklabel(&devices[selected_device],type);
	gtk_widget_hide(TableWindow);
}

void
on_CreateWindow_show                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
	untoggle_create_flags();
	gtk_option_menu_set_history(GTK_OPTION_MENU(lookup_widget(CreateWindow,"optionmenu4")),0);
	gtk_widget_show(lookup_widget(CreateWindow,"CreatePrimaryRadiobutton"));
	gtk_widget_show(lookup_widget(CreateWindow,"CreateExtendedRadiobutton"));
	gtk_widget_show(lookup_widget(CreateWindow,"CreateLogicalRadiobutton"));

	/*
	  can only have one extended partition
	*/
	if (part_extended_nr != 0) {
		gtk_widget_hide(lookup_widget(CreateWindow,"CreateExtendedRadiobutton"));
	}
	/*
	  if you don't have an extended partition,
	  you can't make logical ones
	*/
	else {
		gtk_widget_hide(lookup_widget(CreateWindow,"CreateLogicalRadiobutton"));
	}
	/*
	  if you have enough primary partitions, you can't make any more
	  probably a bug when you have free space left...
	*/
	if (pm_has_max_primary_partitions(&devices[selected_device])) {
		gtk_widget_hide(lookup_widget(CreateWindow,"CreateExtendedRadiobutton"));
		gtk_widget_hide(lookup_widget(CreateWindow,"CreatePrimaryRadiobutton"));
	}
	/* gotta check this one. Will a partition after the extended always
	   be a logical partition? probably not... */
	if (selected_partition > part_extended_nr && part_extended_nr != 0) {
		gtk_widget_hide(lookup_widget(CreateWindow,"CreatePrimaryRadiobutton"));
	}
	if (GTK_WIDGET_VISIBLE(lookup_widget(CreateWindow,"CreatePrimaryRadiobutton"))) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreatePrimaryRadiobutton")),TRUE);
	}
	else if (GTK_WIDGET_VISIBLE(lookup_widget(CreateWindow,"CreateExtendedRadiobutton"))) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateExtendedRadiobutton")),TRUE);
	}
	else if (GTK_WIDGET_VISIBLE(lookup_widget(CreateWindow,"CreateLogicalRadiobutton"))) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateLogicalRadiobutton")),TRUE);
	}
}


void
on_CreatePrimaryRadiobutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton)) {
		show_create_flags();
		show_create_fstype();
	}
}


void
on_CreateExtendedRadiobutton_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton)) {
		hide_create_flags();
		hide_create_fstype();
		gtk_window_resize(GTK_WINDOW(CreateWindow),1,1);
	}
}


void
on_CreateLogicalRadiobutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton)) {
		show_create_flags();
		show_create_fstype();
		gtk_window_resize(GTK_WINDOW(CreateWindow),1,1);
	}
}

void
on_CreateTypeRadiobutton_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *radio_primary_type = lookup_widget(CreateWindow,"CreatePrimaryRadiobutton");
	GtkWidget *radio_extended_type = lookup_widget(CreateWindow,"CreateExtendedRadiobutton");
	GtkWidget *radio_logical_type = lookup_widget(CreateWindow,"CreateLogicalRadiobutton");

	/*
	if (part_primary_count + part_extended_count >= 4) {
		gtk_widget_set_sensitive(radio_primary_type,FALSE);
		gtk_widget_set_sensitive(radio_extended_type,FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_logical_type),TRUE);
	*/

	GtkWidget *optionmenu_fs  = lookup_widget(CreateWindow,"optionmenu4");
	GtkWidget *optionmenu_lbl = lookup_widget(CreateWindow,"label27");

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_primary_type))) {
		gtk_widget_set_sensitive(optionmenu_fs,  TRUE);
		gtk_widget_set_sensitive(optionmenu_lbl, TRUE);
	} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_extended_type))) {
		gtk_widget_set_sensitive(optionmenu_fs,  FALSE);
		gtk_widget_set_sensitive(optionmenu_lbl, FALSE);
	} else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_logical_type))) {
		gtk_widget_set_sensitive(optionmenu_fs,  TRUE);
		gtk_widget_set_sensitive(optionmenu_lbl, TRUE);
	}

}

void
on_WizardWindow_show                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
	// maybe to wizard_ui.c?
	GtkWidget *rad_fs = lookup_widget(WizardWindow,"radiobuttonFreespaceWizard");
	GtkWidget *lab_fs = lookup_widget (WizardWindow,"labelFreespaceWizard");
	GtkWidget *rad_rz = lookup_widget (WizardWindow,"radiobuttonResizeWizard");
	GtkWidget *lab_rz = lookup_widget (WizardWindow,"labelResizeWizard");
	GtkWidget *rad_ds = lookup_widget (WizardWindow,"radiobuttonDestructiveWizard");
	GtkWidget *lab_ds = lookup_widget (WizardWindow,"labelDestructiveWizard");

	GtkWidget *dialog;
	gboolean free_set = FALSE;

	if (return_mode_available(MODE_FREESPACE)) {
		gtk_widget_set_sensitive(rad_fs,TRUE);
		gtk_widget_set_sensitive(lab_fs,TRUE);
		free_set = TRUE;
	}
	else {
		gtk_widget_set_sensitive(rad_fs,FALSE);
		gtk_widget_set_sensitive(lab_fs,FALSE);
	}
	if (return_mode_available(MODE_RESIZE)) {
		if (free_set == FALSE) {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rad_rz),TRUE);
			free_set = TRUE;
		}
		gtk_widget_set_sensitive(rad_rz,TRUE);
		gtk_widget_set_sensitive(lab_rz,TRUE);
	}
	else {
		gtk_widget_set_sensitive(rad_rz,FALSE);
		gtk_widget_set_sensitive(lab_rz,FALSE);
	}
	if (return_mode_available(MODE_DESTRUCTIVE)) {
		if (free_set == FALSE) {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rad_ds),TRUE);
			free_set = TRUE;
		}
		gtk_widget_set_sensitive(rad_ds,TRUE);
		gtk_widget_set_sensitive(lab_ds,TRUE);
	}
	else {
		gtk_widget_set_sensitive(rad_ds,FALSE);
		gtk_widget_set_sensitive(lab_ds,FALSE);
	}
}


void
on_helpbuttonWizard_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_cancelbuttonWizard_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	pm_cleanup();
	gtk_main_quit();
}
// do we need to call pm_cleanup on WizardWindow destroy event ?

void
on_okbuttonWizard_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *rad_fs = lookup_widget(WizardWindow,"radiobuttonFreespaceWizard");
    GtkWidget *rad_rz = lookup_widget (WizardWindow,"radiobuttonResizeWizard");
    GtkWidget *rad_ds = lookup_widget (WizardWindow,"radiobuttonDestructiveWizard");
    GtkWidget *dialog;
    gboolean success = FALSE;

// Check if there are at least one big enought Linux (and swap) partition and suggest to use them.
    if ( can_use_existing() ){
	// Maybe we need to use GTK_DIALOG_DESTROY_WITH_PARENT flag ?
	dialog = gtk_message_dialog_new( GTK_WINDOW (WizardWindow), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
					 GTK_BUTTONS_YES_NO,_("Your disk already contains needed Linux and  swap partitions. Do you want to use existing partitions for Morphix installation?\n") );
	gtk_dialog_set_default_response( GTK_DIALOG(dialog), GTK_RESPONSE_YES );
	gint result = gtk_dialog_run (GTK_DIALOG(dialog));
	if ( result != GTK_RESPONSE_NO ){
	    gtk_widget_destroy (GTK_WIDGET(dialog));
	    pm_cleanup();
	    gtk_main_quit();
	}
	gtk_widget_destroy (GTK_WIDGET(dialog));
    }
	  
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rad_fs)) ) 
	    success = wizard_apply_mode(MODE_FREESPACE);
    else if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rad_rz)) )
	    success = wizard_apply_mode(MODE_RESIZE);
    else if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rad_ds)) ){
	// Maybe we need to use GTK_DIALOG_DESTROY_WITH_PARENT flag ?
        dialog = gtk_message_dialog_new( GTK_WINDOW (WizardWindow), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
                                         GTK_BUTTONS_YES_NO,_("Choosing this option will remove all data on the selected harddisk. You are advised to make a backup of all critical information before continuing.\nDo you have any important data on the selected harddisk?") );
	gtk_dialog_set_default_response( GTK_DIALOG(dialog), GTK_RESPONSE_YES );
        gint result = gtk_dialog_run (GTK_DIALOG (dialog));
        switch (result) {
            case GTK_RESPONSE_NO:
                pm_debug("trina\n");
		success = wizard_apply_mode(MODE_DESTRUCTIVE);
                break;
            default:
                pm_debug("netrina;\n");
                break;
            }
	}
    else
	pm_debug("Error: can't determine selected mode\n");
    if (success == FALSE) {
		/* FIXME: display error dialog */
    }
    pm_cleanup();
    gtk_main_quit();
}


void
on_WarningWindowCancel_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_WarningWindowOk_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{

}

