#include <stdio.h>
#include <assert.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <gtk/gtk.h> // still needs to be built against gtk, as pm_libparted is not completly glib-only...
#include "pm_libparted.h"
#include "pm_fswrap.h"
#include "pm_wrap.h"

GtkWindow *ProgressWindow; // sucks
void show_busy_cursor( GdkWindow * window ) {} // sucks even more
GtkWidget*
lookup_widget                          (GtkWidget       *widget,
                                        const gchar     *widget_name) { } // brrrr...

#define BC_GRUBTEMPLATE1 \
"# See www.gnu.org/software/grub for details\n" \
"# By default, boot the first entry\n" \
"default 0\n" \
"# Boot automatically after 5 seconds\n" \
"timeout 5\n" \
"title Morphix GNU/Linux\n" \
"# kernel path-to-kernel root=rootdevice kernelarguments\n" \
"root %s\n" \
"kernel %s root=%s %s\n"

#define BC_GRUBTEMPLATE_LINUX \
"title GNU/Linux\n" \
"# kernel path-to-kernel root=rootdevice kernelarguments\n" \
"root %s\n" \
"kernel /boot/vmlinuz root=%s %s\n\n"

#define BC_GRUBTEMPLATE_DOS \
"title Windows\n" \
"rootnoverify (hd%d,%d)\n" \
"chainloader +1\n" \
"makeactive\n\n"

#define BC_GRUBTEMPLATE_DOS_SEC_HD \
"title Windows\n" \
"# Not on first harddisk, map the devices to make windows think it is\n" \
"map (hd%d) (hd%d)\n" \
"map (hd%d) (hd%d)\n" \
"rootnoverify (hd%d,%d)\n" \
"chainloader +1\n" \
"makeactive\n\n"

int device_nr;

gboolean read_in_partitions() {
	pm_set_debug(FALSE);
	if (!init_arch())
		return FALSE;
	pm_scan_devices();
	return TRUE;
}

void write_out_menu(char *boot,char *root,char *append_opts,char *image,char *initrd) {
	int i;
	GString *menu = g_string_new("");
	int device_nr, success = 0;
	for (device_nr = 0; device_nr < device_count; device_nr++) {
		pm_select_update_device(device_nr);
		for (i = 0; i < part_count; i++) {
			if (strcmp(boot,partinfo[i].partname) == 0) {
				gchar *root_str = g_strdup_printf("(hd%d,%d)",device_nr,i);
				g_string_append_printf(menu,BC_GRUBTEMPLATE1,root_str,image,root,append_opts);
				if (initrd != NULL) {
					g_string_append_printf(menu,"initrd %s\n",initrd);
				}
				success = 1;
				break;
			}
		}
		if (success == 1)
			break;
		for (i = 0; i < part_count; i++) {
			if ((strcmp(root,partinfo[i].partname) == 0) &&
			    (strcmp(boot,root) != 0) &&
			    (strncmp(boot,root,3) == 0)) { /* you're not supposed to understand this. Well, it would be handy... */
				gchar *root_str = g_strdup_printf("(hd%d,%d)",device_nr,i);
				g_string_printf(menu,BC_GRUBTEMPLATE1,root_str,image,root,append_opts);
				if (initrd != NULL) {
					g_string_append_printf(menu,"initrd %s\n",initrd);
				}
				success = 0;
				break;
			}
		}
		if (success == 1)
			break;
	}
	g_string_append(menu,"\n");

	for (device_nr = 0;device_nr < device_count; device_nr++) {
		pm_select_update_device(device_nr);
		for (i = 0; i < part_count; i++) {
			if (strcmp(root,partinfo[i].partname) == 0) {
				continue; // skip root partition, already added it
			}
			if ((strcmp(partinfo[i].temp_fs_name,"ext2") == 0) ||
			(strcmp(partinfo[i].temp_fs_name,"ext3") == 0) ||
			(strcmp(partinfo[i].temp_fs_name,"reiserfs") == 0) ||
			(strcmp(partinfo[i].temp_fs_name,"xfs") == 0)) {
				gchar *root_str = g_strdup_printf("(hd%d,%d)",device_nr,i);
				g_string_append_printf(menu,BC_GRUBTEMPLATE_LINUX,root_str,partinfo[i].partname,append_opts);
			}	
			if (device_nr != 0) {
				// Windows can't boot from anything but the first harddisk
				// map the selected harddisk onto the first one, 
				// and vice versa
				if ((strcmp(partinfo[i].temp_fs_name,"vfat") == 0) ||
				   (strcmp(partinfo[i].temp_fs_name,"ntfs") == 0)) {
					g_string_append_printf(menu,BC_GRUBTEMPLATE_DOS_SEC_HD,0,device_nr,device_nr,0,0,0,partinfo[i].partname);	
					// swap hd0 -> hddev_nr, hddev_nr->hd0
					// and use hd0,0 as root for booting
				}
			}
			else {
				if ((strcmp(partinfo[i].temp_fs_name,"vfat") == 0) ||
				(strcmp(partinfo[i].temp_fs_name,"ntfs") == 0)) {
					g_string_append_printf(menu,BC_GRUBTEMPLATE_DOS,device_nr,i);			
				}
			}
			pm_debug("type: %s\n",partinfo[i].temp_fs_name);
		}
	}
	printf("%s\n",menu->str);
	g_string_free(menu,TRUE);
}

int main(int argc, char **argv) {
#define ARGMAX 5
  int installed_mbr = 0, fix_mapping = 0;
  int res;
  int boot_number=0;
//  char *device="";
  char *boot=NULL;
  char *root=NULL;
  char *append_opts=NULL;
  char *image=NULL;
  char *initrd=NULL;

// Because argc is always > 1, even with no parameters
  if(argc<ARGMAX)
  {
     fprintf(stderr, "Usage: gengrubconf BootpartitionOrMBRpath Rootpartitionpath AppendOpts KernelImage [initrdpath]\n"
            "Purpose: generate a menu.lst file scanning all partitions\n"
            "and putting the bootable into the grub menu.\n");
     return 1;
  }
  boot = argv[1];
  root = argv[2];
  append_opts=argv[3];
  image = argv[4];
  if (argc > ARGMAX) {
	initrd = argv[5];
  }
  read_in_partitions();
  write_out_menu(boot,root,append_opts,image,initrd);
  /* free partitions... */

}
