/* Copyright (C) 2004 Alex de Landgraaf <alextreme@xs4all.nl>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>

#include "interface.h"
#include "support.h"
#include "pm_libparted.h"

#define PRINT_DEBUGGING_INFO

GtkWidget *MainWindow;
GtkWidget *CreateWindow;
GtkWidget *ResizeWindow;
GtkWidget *ApplyWindow;
GtkWidget *RevertWindow;
GtkWidget *DialogWindow;
GtkWidget *HelpWindow;
GtkWidget *DeleteWindow;
GtkWidget *ConvertWindow;
GtkWidget *MoveWindow;
GtkWidget *TableWindow;
GtkWidget *ProgressWindow;
GtkWidget *WizardWindow;
// I think we should get rid of this fucking ErrorWindow, as we already removed
// WarningWindow and use simple gtk_message_dialog_new () for warnings
GtkWidget *ErrorWindow;
GtkWidget *dialog;

gboolean CheckRoot() {
	if (geteuid() != 0) {
		pm_debug("You must be root to use this tool!\n");
		return FALSE;
	}
	return TRUE;
}

void print_help() {
	printf("Usage\npartitionmorpher [OPTIONS]\n\nOptions:\n-w DRIVE SIZE - start Wizard interface to partition DRIVE to gain minimum space SIZE, which must be > 500MiB, split between root and \n\nIf no options are given, partitionmorpher will start in default (expert) mode\n");
}

int
main (int argc, char *argv[])
{

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	gtk_window_set_auto_startup_notification (FALSE);

	add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

	if (CheckRoot() != TRUE) {
		//gksu_init(argc,argv);
		//gksu_run(argv[0]);
		fprintf(stderr,"Sorry, you need to be root to use partitionmorpher\n");
		return 1;
	}
	#ifdef PRINT_DEBUGGING_INFO
	pm_set_debug(TRUE);
	#else
	pm_set_debug(FALSE);
	#endif

	init_arch();

	pm_scan_devices();

	/* Parted has a bug where if stdout is redirected at the
	   command line to a file, then it can no longer call the
	   _timer_handler with the appropriate amount of time left.
	   Thus, we manually redirect stdout to stderr here */
	dup2(fileno(stderr), fileno(stdout));

	if (argc == 2) { // use the given drive instead
		/* FIXME: after starting, select this device */
// I don't know why -w was here :(
//		if (strcmp(argv[1],"-w") == 0) {
		    gint device = pm_get_valid_device(argv[1]);
		    if (device < 0) {
			pm_debug("Error: Invalid drive\n");
			print_help();
			return 1;
		    }
		    pm_select_update_device(device);
//		}
	}
	else if (argc == 4) { // have three arguments: -w DRIVE SIZE
	    if (strcmp(argv[1],"-w") == 0) {
		gint device = pm_get_valid_device(argv[2]);
		gint size = atoi(argv[3]);
		if (device < 0) {
		    pm_debug("Error: Invalid drive\n");
		    print_help();
		    return 1;
		}
		set_device_nr(device);
		pm_select_update_device(device);
		if (size > selected_device_total_size) {
		    pm_debug("Error: size is larger than the total available space: %d\n",size);
		    print_help();
		    dialog = gtk_message_dialog_new( NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
		                                    GTK_BUTTONS_OK,_("There is not enough space (%d Mb is needed) for automatic partitioning.\n You should partition manually"), size );
		    gtk_window_set_position ( GTK_WINDOW(dialog), GTK_WIN_POS_CENTER );
		    gtk_dialog_run ( GTK_DIALOG(dialog) );
		    gtk_widget_destroy ( GTK_WIDGET(dialog) );
// currently simply will go into expert mode
//			return 1;
		}
		else {
		    set_needed_freespace(size);
		// Check if there are at least one big enought Linux (and swap) partition and suggest to use them.
		    if ( can_use_existing() ){
		        // Maybe we need to use GTK_DIALOG_DESTROY_WITH_PARENT flag ?
			dialog = gtk_message_dialog_new( GTK_WINDOW (WizardWindow), GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,_("Your disk already contains needed Linux and  swap partitions. Do you want to use existing partitions for Morphix installation?\n") );
			gtk_dialog_set_default_response( GTK_DIALOG(dialog), GTK_RESPONSE_YES );
			gtk_window_set_position ( GTK_WINDOW(dialog), GTK_WIN_POS_CENTER );
			gint result = gtk_dialog_run ( GTK_DIALOG(dialog) );
			if ( result != GTK_RESPONSE_NO ){
			        gtk_widget_destroy (GTK_WIDGET(dialog));
				return 0;
			}
			gtk_widget_destroy (GTK_WIDGET(dialog));
		    }
							    	   	
		    // start wizard-interface instead
		    WizardWindow = create_WizardWindow();
		    ProgressWindow = create_ProgressWindow ();
		    gtk_widget_show(WizardWindow);
		    gtk_main();
		    return 0;
		}
	    }
	    else {
		print_help();
		return 1;
	    }
	}
	else if (argc > 2) {
		print_help();
		return 1;
	}

	MainWindow = create_MainWindow ();
	gtk_widget_show (MainWindow);
// Main problem with these Windows is, that they are independant of MainWindow
// so inactive MainWindow can be above active CreateWindows or ErrorWindow
// this confuses lot of users
	CreateWindow = create_CreateWindow ();
	ResizeWindow = create_ResizeWindow ();
//	ApplyWindow = create_ApplyWindow ();
//	RevertWindow = create_RevertWindow ();
//	DialogWindow = create_DialogWindow ();
	HelpWindow = create_HelpWindow ();
	DeleteWindow = create_DeleteWindow ();
	ConvertWindow = create_ConvertWindow ();
	MoveWindow = create_MoveWindow ();
	TableWindow = create_TableWindow ();
	ErrorWindow = create_ErrorWindow ();
	ProgressWindow = create_ProgressWindow ();

  gtk_main ();
  return 0;
}

