/* 
	Copyright (C) 2003 Alex de Landgraaf <alextreme@xs4all.nl>

  Do note that I've used (large) parts of parted.c in parted 
  and qtparted's qp_libparted for this. qp_libparted is in turn largely based on parted.c
  Viva la revolution!

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include "pm_libparted.h"
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>
#include <unistd.h>
#include <parted/parted.h>
#include <parted/debug.h>
#include <parted/linux.h>

#include "support.h"
#include "main.h" /* for debug() */

extern  GtkWidget * ProgressWindow; // should change this, gtk-independant...

#if ENABLE_NLS
#  include <locale.h>
#  include <libintl.h>
#  define _(String) dgettext (PACKAGE, String)
#else
#  define _(String) (String)
#endif /* ENABLE_NLS */

typedef struct {
	time_t	last_update;
	time_t	predicted_time_left;
} TimerContext;

static PedTimer* timer;
static TimerContext timer_context;

//static PedDevice *devices[256] = {NULL};
static gint	opt_script_mode;
static gint 	debug;


extern void ped_disk_bsd_init ();
extern void ped_disk_gpt_init ();
extern void ped_disk_loop_init ();
extern void ped_disk_mac_init ();
extern void ped_disk_mips_init ();
extern void ped_disk_msdos_init ();
extern void ped_disk_pc98_init ();
extern void ped_disk_sun_init ();

void pm_set_debug(gboolean deb) {
	debug = deb;
}

int pm_debug(const char *fmt, ...) {
        int d;
        va_list args;

	if (debug == FALSE) {
		return 0;
	}

        va_start(args, fmt);
                d = vfprintf(stdout,fmt,args);
                va_end(args);

                return d;

        va_end(args);
        return 0;
}

static void
init_disk_types () /* not used yet */
{
	ped_disk_loop_init ();	/* must be last in the probe list */

	ped_disk_sun_init ();
#ifdef ENABLE_PC98
	ped_disk_pc98_init ();
#endif
	ped_disk_msdos_init ();
#ifdef ENABLE_MIPS
	ped_disk_mips_init ();
#endif
	ped_disk_mac_init ();
	ped_disk_gpt_init ();
	ped_disk_bsd_init ();
}

#ifdef ENABLE_FS
extern void ped_file_system_xfs_init (void);
extern void ped_file_system_ufs_init (void);
extern void ped_file_system_reiserfs_init (void);
extern void ped_file_system_ntfs_init (void);
extern void ped_file_system_linux_swap_init (void);
extern void ped_file_system_jfs_init (void);
extern void ped_file_system_hfs_init (void);
extern void ped_file_system_fat_init (void);
extern void ped_file_system_ext2_init (void);

static void
init_file_system_types ()
{
	ped_file_system_xfs_init ();
	ped_file_system_ufs_init ();
	ped_file_system_reiserfs_init ();
	ped_file_system_ntfs_init ();
	ped_file_system_linux_swap_init ();
	ped_file_system_jfs_init ();
	ped_file_system_hfs_init ();
	ped_file_system_fat_init ();
	ped_file_system_ext2_init ();
}
#endif /* ENABLE_FS */

extern void ped_disk_bsd_done ();
extern void ped_disk_gpt_done ();
extern void ped_disk_loop_done ();
extern void ped_disk_mac_done ();
#ifdef ENABLE_MIPS
extern void ped_disk_mips_done ();
#endif
extern void ped_disk_msdos_done ();
extern void ped_disk_pc98_done ();
extern void ped_disk_sun_done ();

static void
done_disk_types () /* not used yet */
{
	ped_disk_sun_done ();
#ifdef ENABLE_PC98
	ped_disk_pc98_done ();
#endif
	ped_disk_msdos_done ();
#ifdef ENABLE_MIPS
	ped_disk_mips_done ();
#endif 
	ped_disk_mac_done ();
	ped_disk_loop_done ();
	ped_disk_gpt_done ();
	ped_disk_bsd_done ();
}

/* This sucks.  I really dislike using extern, but...  well, it was
   already used quite a bit anyway... */
extern void show_busy_cursor( GdkWindow * window );

static void
_timer_handler (PedTimer* timer, void* context);

// why this usefull function is in pm_libparted? why it was static?
// It would be very useful to use it in  do_mkext3 and others
void display_progress_window()
{

	GtkWidget *progress_label = lookup_widget(ProgressWindow,"ProgressLabel");
	GtkWidget *progress_bar = lookup_widget(ProgressWindow,"ProgressBar");
	gtk_label_set_text(GTK_LABEL(progress_label), "Estimating remaining time...");
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress_bar), 0.0);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress_bar), "0%");
	gtk_widget_show(ProgressWindow);

	show_busy_cursor( ProgressWindow->window );

	while (gtk_events_pending())
		gtk_main_iteration();
}

static void
_timer_handler (PedTimer* timer, void* context)
{
	TimerContext*	tcontext = (TimerContext*) context;
	int		draw_this_time;

	if (opt_script_mode || !isatty(fileno(stdout)))
		return;

	if (tcontext->last_update != timer->now && timer->now > timer->start) {
		tcontext->predicted_time_left
			= timer->predicted_end - timer->now;
		tcontext->last_update = timer->now;
		draw_this_time = 1;
	} else {
		draw_this_time = 0;
	}

	if (draw_this_time) {

		/* The timer seems to always hit 0 a second or two
		   before quitting.  So, I'm just going to increment
		   it here.  On my laptop, it needs a bigger increment
		   than this, but at least it doesn't show "00:00" as
		   the time left and still sit there for a couple
		   seconds.  Of course, the timer in general seems to
		   need some work. */
		tcontext->predicted_time_left++;

		//	wipe_line ();
		if (timer->state_name)
			pm_debug ("%s... ", timer->state_name);
		pm_debug (_("%0.f%%\t(time left %.2d:%.2d)\n"),
			100.0 * timer->frac,
			tcontext->predicted_time_left / 60,
			tcontext->predicted_time_left % 60);
		fflush (stdout);

		gchar message[100];
		gchar percentage[10];
		sprintf(message,"");
		if (timer->state_name) {
			sprintf (message,"%s\n", timer->state_name);
			/* The following is an ugly hack to make the first character upperase. */
			message[0] &= 0xDF;
		}
		sprintf (message, _("%sTime left %.2ld:%.2ld\n"),
			message,
			tcontext->predicted_time_left / 60,
			tcontext->predicted_time_left % 60);
		sprintf(percentage, "%0.f%%", 100.0 * timer->frac);
		
		GtkWidget *progress_label = lookup_widget(ProgressWindow,"ProgressLabel");
		GtkWidget *progress_bar = lookup_widget(ProgressWindow,"ProgressBar");
		gtk_label_set_text(GTK_LABEL(progress_label), message);
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress_bar), timer->frac);
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progress_bar), percentage);
		while (gtk_events_pending())
			gtk_main_iteration();

	}
}

gboolean set_timer() {
	timer = ped_timer_new (_timer_handler, &timer_context);
	if (!timer) { 
		pm_debug("no timer!\n");
		return FALSE;
	}
	timer_context.last_update = 0;
	return TRUE;
}

gboolean init_arch() {
#ifdef ENABLE_NLS
	//bindtextdomain (PACKAGE, LOCALEDIR);
#endif

	//init_disk_types ();

#ifdef ENABLE_FS
	init_file_system_types ();
#endif

	/* Only works on linux-arch now. Sue me */
  	ped_set_architecture (&ped_linux_arch);

	return set_timer();
}

static inline PedSector
_ped_abs (PedSector sector)
{
	return sector > 0 ? sector : -sector;
}

/*
 * FIXME! This check has been disabled, like in qtparted,
 * but it's common sense not to leave this like this
 */

gboolean partition_warn_busy (PedPartition *part, gchar *path2) { /* needs fixing! */
/*	char *path = ped_partition_get_path (part);
	char *path = malloc(strlen(path2)+1);
	strcpy (path,path2);
	if (ped_partition_is_busy(part)) {
		printf("Partition is being used!\n");
		ped_free(path);
		return FALSE;
	}
	ped_free(path);
*/	return TRUE;
}

static int
disk_warn_busy (PedDisk* disk)
{
	if (ped_device_is_busy (disk->dev)) {
		if (ped_exception_throw (
			PED_EXCEPTION_WARNING,
			PED_EXCEPTION_IGNORE_CANCEL,
			"Partition(s) on %s are being used.",
			disk->dev->path)
				!= PED_EXCEPTION_IGNORE)
			return 0;
	}
	return 1;
}

int
pm_solution_check_distant (PedSector req_start, PedSector req_end,
			PedSector soln_start, PedSector soln_end) {
	PedSector	start_delta = _ped_abs (soln_start - req_start);
	PedSector	end_delta = _ped_abs (soln_end - req_end);
	PedSector	distance = start_delta + end_delta;
	float		distance_frac;

	distance_frac = PED_MAX (1.0 * distance / (req_end - req_start + 1),
				 1.0 * distance / (soln_end - soln_start + 1));

	if (distance == -1
	    || (distance > 10 * MEGABYTE_SECTORS && distance_frac > 0.1)) {
		fprintf(stderr, "Can't reasonably fit within specified constraints\n");
		return FALSE;
	}

/*	if (ped_exception_throw (
			PED_EXCEPTION_WARNING,
			PED_EXCEPTION_OK_CANCEL,
			msg,
			req_start * 1.0 / MEGABYTE_SECTORS,
			req_end * 1.0 / MEGABYTE_SECTORS,
			soln_start * 1.0 / MEGABYTE_SECTORS,
			soln_end * 1.0 / MEGABYTE_SECTORS)
				== PED_EXCEPTION_CANCEL)
				return 0;
				}*/

	return 1;
}

static PedSector get_left_bound(PedSector sector, PedDisk *disk) {
	PedPartition *under_sector;

	under_sector = ped_disk_get_partition_by_sector (disk,sector);
	PED_ASSERT (under_sector != NULL, return sector);
	
	if (under_sector->type & PED_PARTITION_FREESPACE)
		return under_sector->geom.start;
	return sector;
}

static PedSector get_right_bound(PedSector sector, PedDisk *disk) {
	PedPartition *under_sector;

	under_sector = ped_disk_get_partition_by_sector (disk,sector);
	PED_ASSERT (under_sector != NULL, return sector);
	
	if (under_sector->type & PED_PARTITION_FREESPACE)
		return under_sector->geom.end;
	return sector;
}

gboolean grow_over_small_freespace(PedGeometry *geom, PedDisk *disk) {
	PedSector start;
	PedSector end;
	
	if (geom->length < MIN_FREESPACE *5)
		return FALSE;
	start = get_left_bound (geom->start,disk);
	PED_ASSERT (start < geom->end, return FALSE);
	if (geom -> start - start < MIN_FREESPACE)
		ped_geometry_set_start (geom, start);
	
	end = get_right_bound(geom->end,disk);
	PED_ASSERT(end > geom->start, return FALSE);
	if (end - geom->end < MIN_FREESPACE)
		ped_geometry_set_end(geom,end);
	return TRUE;
}

gboolean pm_do_mklabel (PedDevice **dev, PedDiskType *type) {
	PedDisk *disk;

	ped_exception_fetch_all();
	disk = ped_disk_new(*dev);
	if (!disk) 
		ped_exception_catch();
	ped_exception_leave_all();

	if (disk) {
		if (!disk_warn_busy(disk)) {
			ped_disk_destroy(disk);
			return FALSE;
		}
		ped_disk_destroy(disk);
	}
	if (!type)
		return FALSE;
	disk = ped_disk_new_fresh(*dev,type);
	
	if (!disk)
		return FALSE;

	if (!ped_disk_commit(disk)) {
		ped_disk_destroy(disk);
		return FALSE;
	}
	ped_disk_destroy(disk);
	return TRUE;
}
	
gboolean pm_do_mkfs (PedDevice **dev, int part_nr ,gchar *path, PedFileSystemType *type) {
	PedDisk *disk;
	PedFileSystem *fs;
	PedPartition *part;

	disk = ped_disk_new (*dev);
	if (!disk || !path) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	part = ped_disk_get_partition(disk,part_nr);
	if (part == NULL) {
		return FALSE;
	}
	if (!partition_warn_busy(part,path)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	display_progress_window();
	fs = ped_file_system_create (&part->geom,type,timer);
	gtk_widget_hide(ProgressWindow);

	if (!fs) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_file_system_close(fs);
	
	if (!ped_partition_set_system(part,type)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_disk_commit(disk)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_disk_destroy(disk);
	return TRUE;
}
	
gboolean pm_do_mkpart (PedDevice **dev,PedPartitionType part_type, PedSector start, PedSector end) {
	PedDisk *disk;
	PedPartition *part;
	PedConstraint *constraint;
	//char *peek_word;
	PedFileSystemType *fs_type = NULL;

	disk = ped_disk_new(*dev);
	if (!disk) 
		return FALSE;
	constraint = ped_constraint_any(*dev);
	if (!constraint) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	//peek_word = command_line_peek_word();
	part = ped_partition_new(disk,part_type,fs_type,start,end);
	if (!part) {
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!grow_over_small_freespace(&part->geom,disk)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_disk_add_partition(disk,part,constraint)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}

	if (!pm_solution_check_distant(start,end,part->geom.start,part->geom.end)) {
		ped_disk_remove_partition (disk, part);
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	
	if (!ped_partition_set_system(part,fs_type)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}

	if (!ped_disk_commit(disk)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_constraint_destroy(constraint);
	ped_disk_destroy(disk);
	return TRUE;
}

gboolean pm_do_mkpartfs (PedDevice **dev, PedPartitionType part_type, 
		      PedFileSystemType *fs_type, 
		      PedSector start, PedSector end) {
	PedDisk *disk;
	PedConstraint *constraint;
	PedPartition *part;
	PedFileSystem *fs;

	disk = ped_disk_new (*dev);
	if (!disk)
		return FALSE;
	if (part_type == PED_PARTITION_EXTENDED) {
		pm_debug("Extended partitions can't have file systems.\nDid you want mkpart?\n");
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!fs_type) {
		pm_debug("No filesystem specified; application should probably call mkpart instead of mkpartfs\n");
		ped_disk_destroy (disk);
		return FALSE;	
	}
	constraint = ped_file_system_get_create_constraint (fs_type, *dev);
	part = ped_partition_new (disk, part_type, fs_type, start, end);
	if (!part) {
		ped_constraint_destroy(constraint);
		ped_disk_destroy(disk);
		return FALSE;
	}
	if (!grow_over_small_freespace(&part->geom,disk)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_disk_add_partition (disk,part,constraint)) {
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!pm_solution_check_distant(start,end,part->geom.start,part->geom.end)) {
		ped_disk_remove_partition (disk, part);
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	display_progress_window();
	fs = ped_file_system_create(&part->geom, fs_type, timer);
	gtk_widget_hide(ProgressWindow);
	if (!fs) {
		ped_disk_remove_partition (disk, part);
		ped_partition_destroy (part);	
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	
	ped_file_system_close (fs);
	ped_constraint_destroy (constraint);

	if (!ped_partition_set_system(part,fs_type)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_disk_commit(disk)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_disk_destroy(disk);
	return TRUE;
}
		
gboolean pm_do_move(PedDevice **dev, int part_nr,gchar *path, PedSector start, PedSector end) {
	PedDisk *disk;
	PedFileSystem *fs;
	PedFileSystem *fs_copy;
	PedConstraint *constraint;
	PedGeometry old_geom;
	PedGeometry new_geom;
	PedPartition *part;

	disk = ped_disk_new(*dev);
	if (!disk) {
		return FALSE;
	}
	part = ped_disk_get_partition(disk,part_nr);
	if (!part) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!partition_warn_busy(part,path)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (part->type == PED_PARTITION_EXTENDED) {
		pm_debug("Can't move extended partitions!\n");
		ped_disk_destroy (disk);
		return FALSE;
	}
	old_geom = part->geom;

	fs = ped_file_system_open (&old_geom);
	if (!fs) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	constraint = ped_file_system_get_copy_constraint(fs,*dev);
	
	if (!ped_geometry_init (&new_geom, *dev, start, end - start + 1)) {
		ped_constraint_destroy (constraint);
		ped_file_system_close (fs);
		ped_disk_destroy (disk);
		return FALSE;
	}

	if (!grow_over_small_freespace(&new_geom,disk)) {
		ped_constraint_destroy (constraint);
		ped_file_system_close (fs);
		ped_disk_destroy (disk);
		return FALSE;
	}
	
	if (!ped_disk_set_partition_geom(disk,part,constraint,new_geom.start, new_geom.end)) {
		ped_constraint_destroy (constraint);
		ped_file_system_close (fs);
		ped_disk_destroy (disk);
		return FALSE;
	}
	
	ped_constraint_destroy(constraint);

	if (ped_geometry_test_overlap (&old_geom, &part->geom)) {
		pm_debug("Can't move a partition onto itself, resize maybe?\n");
		ped_file_system_close (fs);
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!pm_solution_check_distant(start,end,part->geom.start, part->geom.end)) {
		ped_file_system_close (fs);
		ped_disk_destroy (disk);
		return FALSE;
	}
	display_progress_window();
	fs_copy = ped_file_system_copy (fs, &part->geom, timer);
	gtk_widget_hide(ProgressWindow);
	if (!fs_copy) {
		ped_file_system_close (fs);
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_file_system_close(fs_copy);
	ped_file_system_close(fs);
	if(!ped_disk_commit(disk)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_disk_destroy(disk);
	return TRUE;
}

gboolean pm_do_name(PedDevice **dev,int part_nr, gchar *name) {
	PedDisk *disk;
	PedPartition *part;
	if (!pm_disk_supports_name (dev)) {
		return FALSE;
	}
	disk = ped_disk_new(*dev);
	if (!disk) {
		return FALSE;
	}
	part = ped_disk_get_partition(disk,part_nr);
	if (!name) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_partition_set_name(part,name)) {
		ped_disk_destroy(disk);
		return FALSE;
	}
	if (!ped_disk_commit(disk)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_disk_destroy (disk);
	return TRUE;
}

gboolean pm_do_resize (PedDevice **dev, int part_nr ,gchar *path, PedSector start, PedSector end) {
	PedDisk *disk;
	PedFileSystem *fs;
	PedConstraint *constraint;
	PedGeometry new_geom;
	PedPartition *part;

	disk = ped_disk_new(*dev);
	if (!disk) 
		return FALSE;
	part = ped_disk_get_partition(disk,part_nr);
	if (!part) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!partition_warn_busy(part,path)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!start || !end) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_geometry_init (&new_geom, *dev, start, end - start + 1)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
/*	if (!grow_over_small_freespace(&new_geom,disk)) {
		ped_disk_destroy (disk);
		return FALSE;
	}*/
	if (part->type == PED_PARTITION_EXTENDED) {
		constraint = ped_constraint_any(*dev);
		if (constraint == NULL) {
			ped_disk_destroy (disk);
			return FALSE;
		}
		if (!ped_disk_set_partition_geom(disk,part,constraint,
						 new_geom.start, new_geom.end)) {
			ped_constraint_destroy (constraint);
			ped_disk_destroy (disk);
			return FALSE;
		}
		if (!pm_solution_check_distant (start, end, part->geom.start, part->geom.end)) {
			ped_constraint_destroy (constraint);
			ped_disk_destroy (disk);
			return FALSE;
		}
		ped_partition_set_system(part,NULL);
	}
	else {
		fs = ped_file_system_open(&part->geom);
		if (!fs) {
			ped_disk_destroy (disk);
			return FALSE;
		}
		constraint = ped_file_system_get_resize_constraint(fs);
		if (constraint == NULL) {
			ped_file_system_close (fs);
			return FALSE;
		}
			
		if (!ped_disk_set_partition_geom(disk,part,constraint,
						 new_geom.start,new_geom.end)) {
			ped_file_system_close (fs);
			ped_constraint_destroy (constraint);
			ped_disk_destroy (disk);
			return FALSE;
		}
		if (!pm_solution_check_distant(start,end,part->geom.start,
					    part->geom.end)) {
			ped_file_system_close (fs);
			ped_constraint_destroy (constraint);
			ped_disk_destroy (disk);
			return FALSE;
		}
		display_progress_window();
		if (!ped_file_system_resize(fs,&part->geom,timer)) {
			ped_file_system_close (fs);
			ped_constraint_destroy (constraint);
			ped_disk_destroy (disk);
			return FALSE;
		}
		gtk_widget_hide(ProgressWindow);
		ped_partition_set_system (part, fs->type);
		ped_file_system_close(fs);
	}

	ped_disk_commit(disk);
	ped_constraint_destroy(constraint);
	ped_disk_destroy(disk);
	return TRUE;
}

gboolean pm_do_resize_part_only (PedDevice **dev, int part_nr ,gchar *path, PedSector start, PedSector end) {
	PedDisk *disk;
	PedConstraint *constraint;
	PedGeometry new_geom;
	PedPartition *part;

	disk = ped_disk_new(*dev);
	if (!disk) 
		return FALSE;
	part = ped_disk_get_partition(disk,part_nr);
	if (!part) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!partition_warn_busy(part,path)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!start || !end) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_geometry_init (&new_geom, *dev, start, end - start + 1)) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	constraint = ped_constraint_any(*dev);
	if (constraint == NULL) {
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!ped_disk_set_partition_geom(disk,part,constraint,
					 new_geom.start, new_geom.end)) {
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	if (!pm_solution_check_distant (start, end, part->geom.start, part->geom.end)) {
		ped_constraint_destroy (constraint);
		ped_disk_destroy (disk);
		return FALSE;
	}
	ped_partition_set_system(part,NULL);

	ped_disk_commit(disk);
	ped_constraint_destroy(constraint);
	ped_disk_destroy(disk);
	return TRUE;
}


gboolean pm_do_rm(PedDevice **dev, gint num) {
	PedDisk *disk;
	PedPartition *part;
	disk = ped_disk_new(*dev);
	if(!disk)
		return FALSE;
	part = ped_disk_get_partition(disk,num);
	if (part == NULL) {
		return FALSE;
	}
	if (ped_partition_is_busy(part)) {
//	if (!partition_warn_busy(part, path)) {
	      ped_disk_destroy (disk);
	      return FALSE;
	}
	ped_disk_delete_partition(disk,(PedPartition *)part);
	ped_disk_commit(disk);
	ped_disk_destroy(disk);
	return TRUE;
}

gboolean select_device(PedDevice **dev, PedDevice **new_dev) {
	if (!new_dev)
		return FALSE;
	if (!ped_device_open(*new_dev))
		return FALSE;
	ped_device_close(*dev);
	*dev = *new_dev;
	return TRUE;
}

gint pm_get_part_flag_state(PedDevice **dev,int part_nr, PedPartitionFlag flag) {
	PedDisk *disk;
 	PedPartition *part;
	gint state = 0;
	
	disk = ped_disk_new(*dev);
	if (!disk)
		return -1;
	part = ped_disk_get_partition(disk,part_nr);
	if (!part) {
		ped_disk_destroy(disk);
		return -1;
	}
	state = ped_partition_get_flag(part,flag);
	ped_disk_destroy(disk);
	return state;
}



gboolean pm_set_part_flag_state(PedDevice **dev,int part_nr, PedPartitionFlag flag, gint state) {
	PedDisk *disk;
	PedPartition *part;
	disk = ped_disk_new (*dev);
	if (!disk)
		return FALSE;
	part = ped_disk_get_partition(disk,part_nr);
	if (!part) {
		ped_disk_destroy(disk);
		return FALSE;
	}
	if (!ped_partition_set_flag(part,flag,state)) {
		ped_disk_destroy(disk);
		return FALSE;
	}
	if (!ped_disk_commit(disk)) {
		ped_disk_destroy(disk);
		return FALSE;
	}
	ped_disk_destroy(disk);
	return TRUE;
}

gchar *pm_get_flag_name(PedPartitionFlag part_flag) {
	gchar * buffer;
	buffer = malloc(256);
	strncpy(buffer,ped_partition_flag_get_name (part_flag),255);
	return buffer;
}

/*
gchar * partition_get_fs_name(int part_nr) {
	PedDisk *disk;
	PedDevice *dev;
	PedFileSystem *fs;
	gchar *fs_name;
	PedPartition *part;

	disk = ped_disk_new(part->disk->dev);
	if (!disk) {
		printf("Disk Error!\n");
		return NULL;
	}
	part = ped_disk_get_partition(disk,part_nr);
	if (!part) {
		printf("Partition invalid!\n");
	}
	
	fs_name = malloc(256); // Free this when you are done with it
	fs = ped_file_system_open(&part->geom);
	if (!fs) {
		ped_file_system_close(fs);
		ped_disk_destroy(disk);
		return NULL;
	}
	strcpy(fs_name,fs->type->name);
	ped_file_system_close(fs);
	ped_disk_destroy(disk);
	return fs_name;
	}*/

gfloat pm_partition_get_length(PedPartition *part) {
	return part->geom.length * 1.0 / MEGABYTE_SECTORS;
}

gfloat pm_partition_max_resize(PedPartition *part) {
	PedFileSystem *fs;
	PedConstraint *resize_constraint;
	gfloat max_size = -1;

	fs = ped_file_system_open(&part->geom);
	if (!fs)
		return -1;
	resize_constraint = ped_file_system_get_resize_constraint (fs);
	if (resize_constraint) {
		max_size = resize_constraint->max_size * 1.0 / MEGABYTE_SECTORS;
		ped_constraint_destroy(resize_constraint);
	}
	ped_file_system_close(fs);
	return max_size;
}

gfloat pm_partition_min_resize(PedPartition *part) {
	PedFileSystem *fs;
	PedConstraint *resize_constraint;
	gfloat min_size = -1;

	fs = ped_file_system_open(&part->geom);
	if (!fs)
		return -1;
	resize_constraint = ped_file_system_get_resize_constraint (fs);
	if (resize_constraint) {
		min_size = resize_constraint->min_size * 1.0 / MEGABYTE_SECTORS;
		ped_constraint_destroy(resize_constraint);
	}
	ped_file_system_close(fs);
	return min_size;
}

gboolean pm_disk_supports_extended (PedDevice **dev) {
	PedDisk *disk;

	disk = ped_disk_new (*dev);
	if (!disk) 
		return FALSE;

	if (ped_disk_type_check_feature(disk->type,PED_DISK_TYPE_EXTENDED)) {
		ped_disk_destroy (disk);
		return TRUE;
	}
	ped_disk_destroy (disk);
	return FALSE;
}

gboolean pm_disk_supports_name (PedDevice **dev) {
	PedDisk *disk;

	disk = ped_disk_new (*dev);
	if (!disk) 
		return FALSE;

	if (ped_disk_type_check_feature(disk->type,PED_DISK_TYPE_PARTITION_NAME)) {
		ped_disk_destroy (disk);
		return TRUE;
	}
	ped_disk_destroy (disk);
	return FALSE;
}

gboolean pm_use_partition(PedPartition *part) {
	if (part == NULL) {
		/* Non-partition */
		pm_debug("Debug: Skipping NULL partition\n");
		return FALSE;
	}
	if (part->type & PED_PARTITION_METADATA) {
		/* metadata, probably partition table */
		pm_debug("Debug: Not using METADATA partition\n");
		return FALSE;
	}
	return TRUE;
}

/* rescuing partitions is left out for now, check parted.c for more info */

gboolean pm_scan_partitions(PedDevice **dev) {
	PedDisk *disk;
	PedPartition *part = NULL;
	
	gint end = 0;
	part_count = 0;

	disk = ped_disk_new (*dev);
	if (!disk) {
		pm_debug("Error: disk not made in scan_partitions\n");
		return FALSE;
	}
	part = ped_disk_next_partition(disk,part);
	end = ped_disk_get_last_partition_num (disk);
	pm_debug("Debug: Last num: %d\n",end);

	for (;part != NULL; part = ped_disk_next_partition(disk,part)) {
		partitions[part_count] = part;
		pm_debug("Debug: length: %.f\n",pm_partition_get_length(part));
		if (pm_use_partition(part)) {
			part_count++;
		}
	} 
	
	ped_disk_destroy (disk);
	return TRUE;
}

/*
  an attempt to clean out the array before adding other crap to it
*/

void pm_initialise_partinfo(gint i) {
	gint j;

	partinfo[i].type = 0;
	//partinfo[i].part = NULL;
	for (j = 0; j < MAX_FLAGS; j++) {
		partinfo[i].flags[j] = 0;
	}
	partinfo[i].flag_count = 0;
	partinfo[i].t_start = 0;
	partinfo[i].t_end = 0;
	partinfo[i].start = 0;
	partinfo[i].end = 0;
	partinfo[i].max_size = 0;
	partinfo[i].min_size = 0;

	partinfo[i].part_widget = NULL;
	partinfo[i].row_number = 0;

	partinfo[i].fs = NULL;
	strcpy(partinfo[i].temp_fs_name,"");
	strcpy(partinfo[i].temp_flags_list,"");
	partinfo[i].num = -1;
	
	partinfo[i].size = 0;
	partinfo[i].mb_min_size = 0;
	partinfo[i].mb_max_size = 0;
	strcpy(partinfo[i].partname,"");
	partinfo[i].label = NULL;
	partinfo[i].free = FALSE;
	partinfo[i].unknown = FALSE;
	partinfo[i].width = 0;
}

gboolean pm_update_partinfo() {
	PedDisk *disk = NULL;
	PedPartition *part = NULL;
	PedPartitionFlag flag;
	gint i;
	gint flags;
	gchar buffer[64];
	const char *label = NULL;

	disk = ped_disk_new (devices[selected_device]);
	if (!disk) {
		pm_debug("Debug: No partitions!\n");
		
		return FALSE;
	}
	selected_device_freespace = 0;

	set_timer();

	part_extended_nr = part_primary_count = part_extended_count = part_logical_count = part_freespace_count = 0;

	for (i = 0; i < part_count; i++) {
		pm_initialise_partinfo(i);
		part = ped_disk_next_partition(disk,part);
		if (!pm_use_partition(part)) {
			i--;
			continue;
		}
		if (G_IS_OBJECT(partinfo[i].part_widget))
			gtk_container_remove(GTK_CONTAINER(part_widget),partinfo[i].part_widget);
		
		PedConstraint *constraint = NULL;
		if (!part && !disk) {
			pm_debug("Error: reading partition!\n");
			return TRUE;
		}

		//pm_debug(minor,"%d",partition_get_minor(part));

		partinfo[i].start = (int)part->geom.start;
		partinfo[i].end = (int)part->geom.end;
		partinfo[i].num = part->num;
		partinfo[i].size = pm_pedsector_to_mb(partinfo[i].end - partinfo[i].start);
		partinfo[i].label = "";
		if (!(part->type & PED_PARTITION_FREESPACE)) {
			pm_debug("Debug: Adding partition\n");
			strcpy(partinfo[i].partname,ped_partition_get_path(part));
			if (part->type & PED_PARTITION_LOGICAL) {
				partinfo[i].type = LOGICAL;
				part_logical_count++;
			}
			else if (part->type & PED_PARTITION_EXTENDED) {
				partinfo[i].type = EXTENDED;
				part_extended_count++;
				part_extended_nr = i;
			}
			else {
				partinfo[i].type = PRIMARY;
				part_primary_count++;
			}

			flag = ped_partition_flag_next((PedPartitionFlag)0);
			flags = 0;
			strcpy(partinfo[i].temp_flags_list,"");
			while (flag) {
				if (ped_partition_get_flag(part,flag)) {
					partinfo[i].flags[flags] = flag;
					strncpy(buffer,ped_partition_flag_get_name(flag),63);
					strcpy(buffer,strcat(buffer," "));
					strcpy(partinfo[i].temp_flags_list,strcat(partinfo[i].temp_flags_list,buffer));
					
				}
				flag = ped_partition_flag_next(flag);
				
				flags++;
			}
			partinfo[i].free = FALSE;
			constraint = ped_constraint_any(devices[selected_device]);
			PedGeometry *geometry = NULL;
			if (constraint != NULL) 
				geometry = ped_disk_get_max_partition_geometry(disk,part,constraint);
			if (geometry != NULL) {
				partinfo[i].t_start = (int)geometry->start;
				partinfo[i].t_end = (int)geometry->end;
			} else {
				partinfo[i].t_start = -1;
				partinfo[i].t_end = -1;
			}
			partinfo[i].max_size = (int) pm_partition_max_resize(part);
			partinfo[i].min_size = (int) pm_partition_min_resize(part);
			PedFileSystem *fs = ped_file_system_open(&part->geom);
			if (fs) {
/*
				pm_debug("Debug: Checking if partition %s is resizable...\n", partinfo[i].partname);

				PedConstraint *resize_con = 
					ped_file_system_get_resize_constraint(fs);
				if (resize_con) {
					pm_debug("Debug: constraint accepted: mb_min: %ld mb_max: %ld\n",(int)(resize_con->min_size * 1.0/ MEGABYTE_SIZE * MEGABYTE_SECTORS),(int)(resize_con->max_size * 1.0 / MEGABYTE_SIZE * MEGABYTE_SECTORS));
					partinfo[i].min_size = (int)resize_con->min_size;
					partinfo[i].mb_min_size = (int)(resize_con->min_size * 1.0/ MEGABYTE_SIZE * MEGABYTE_SECTORS);
					partinfo[i].max_size = (int)resize_con->max_size;
					partinfo[i].mb_max_size = (int)(resize_con->max_size * 1.0 / MEGABYTE_SIZE * MEGABYTE_SECTORS);
					ped_constraint_destroy(resize_con);
				}
				else {
					pm_debug("Debug: constraint failed! Can'tresize %dth partition\n",i);
				}
*/
				ped_file_system_close(fs);
				// The following two lines shouldn't be necessary since we initialize partinfo[i].label to ""
				//if (partinfo[i].label != NULL)
				//g_free(partinfo[i].label);
				label = ped_partition_get_name(part);
				if (label != NULL) 
					partinfo[i].label = label;
			}
			else {
				pm_debug("Debug: filesystem couldn't be opened\n");
			}
		}
		else {
			/* FREESPACE */
			pm_debug("Debug: Adding freespace\n");
			selected_device_freespace += partinfo[i].size;
			partinfo[i].t_start = -1;
			partinfo[i].t_end = -1;
			partinfo[i].free = TRUE;
			partinfo[i].type = FREE_SPACE;

			/* Unfortunately, ped_disk_next_partition sets part->num to -1
			   for all partitions corresponding to free space.  This
			   causes a small bug: whenever the user highlights some free
			   space and clicks on "Create", they are forced to create a
			   partition corresponding to the first set of free space on
			   the drive regardless of which set of free space they
			   clicked on.  This is a simple workaround for this problem. */
			partinfo[i].num -= part_freespace_count++;
		}

		strcpy(partinfo[i].temp_fs_name,part->fs_type ? part->fs_type->name : "");
		strncpy(partinfo[i].type_name,ped_partition_type_get_name(part->type),63);
	}
	ped_disk_destroy(disk);
	return TRUE;
}

/*
  initialise list of filesystem types
  stores this in fsinfo-array, contains name, color and bool's for cr/re/cp
*/

void pm_get_filesystem_list() {
	PedFileSystemType *walk = ped_file_system_type_get_next(NULL);
	gint i = 0;
	while (walk) {
		fsinfo[i].name = malloc(strlen(walk->name) + 1);
		strcpy(fsinfo[i].name,walk->name);
		fsinfo[i].creatable = (gboolean) walk->ops->create;
		fsinfo[i].resizeable = (gboolean) walk->ops->resize;
		fsinfo[i].copyable  = (gboolean) walk->ops->copy;
		fsinfo[i].color = i;
		walk = ped_file_system_type_get_next(walk);
	}
	fs_count = i;
}

/*
	from a partition number, get the corresponding
	nr of the partinfo nr, or vice versa
*/

gint pm_get_arraynumber_from_partinfo_num(gint num) {
	gint i;
	for (i = 0; i < part_count; i++) {
		if (partinfo[i].num == num) {
			return i;
		}
	}
	return -1;
}

struct partinfo_struct *pm_get_partition_from_partinfo_num(gint num) {
	gint i;
	for (i = 0; i < part_count; i++) {
		if (partinfo[i].num == num) {
			return &partinfo[i];
		}
	}
	return NULL;
}

gboolean pm_has_max_primary_partitions(PedDevice **device) {
	PedDisk *disk;

	disk = ped_disk_new (*device);
	if (!disk) {
		pm_debug("Error: disk not made in has_max_primary_part\n");
		return FALSE;
	}
	if (ped_disk_get_primary_partition_count(disk) ==
	    ped_disk_get_max_primary_partition_count(disk)) {
		ped_disk_destroy (disk);
		pm_debug("Debug: Disk has max primary partitions, count == max\n");
		return TRUE;
	}
		
	ped_disk_destroy (disk);
	return FALSE;
}

gint pm_get_primary_partitions_count(PedDevice **device) {
	PedDisk *disk;
	int count = 0;
	disk = ped_disk_new (*device);
	if (!disk) {
		pm_debug("Error: disk not made in has_max_primary_part\n");
		return -1;
	}
	count = ped_disk_get_primary_partition_count(disk);
	ped_disk_destroy (disk);
	return count;
}
	
gboolean pm_has_extended_partition(PedDevice **device) {
	PedDisk *disk;

	disk = ped_disk_new (*device);
	if (!disk) {
		pm_debug("Error: disk not made in has_extended_part\n");
		return FALSE;
	}	
	if (ped_disk_extended_partition(disk) != NULL) {
		ped_disk_destroy (disk);
		pm_debug("Debug: Device has extended partition\n");
		return TRUE;
	}
	pm_debug("Debug: Device doesn't have extended partition\n");
	ped_disk_destroy (disk);
	return FALSE;
}

/*
	select the item in our devices array
	note that this probably shouldn't be used outside this file,
	as it doesn't update or initialize the partition stuff
*/

void pm_select_device(int device_nr) {
	selected_device = device_nr;
	selected_device_total_size = devices[device_nr]->length * 1.0 / MEGABYTE_SIZE * devices[device_nr]->sector_size;
}

/*
	select the item in our devices array
	scan all partitions of the newly selected device
	update the relevant information in our partinfo array
*/

gboolean pm_select_update_device(int device_nr) {
	pm_select_device(device_nr);
	if (!pm_scan_partitions(&devices[selected_device]))
		return FALSE;
	if (!pm_update_partinfo())
		return FALSE;
	return TRUE;
}

void pm_scan_devices() {
	ped_device_probe_all();	
	PedDevice *dev = pm_get_next_device(NULL);
	device_count = 0;
	while (dev != NULL) {
		devices[device_count] = dev;
		dev = pm_get_next_device(dev);
		device_count++;
	}
	selected_partition = 0;
}

/*
	checks if the device_path is a valid drive
	if it is, return the item in our devices array
	if not, return -1
*/

gint pm_get_valid_device(char* device_path) {
	gint i;
	if (device_path == NULL)
		return -1;
	for (i = 0; i < device_count; i++) {
		if (strcmp(device_path,devices[i]->path) == 0) {
			return i;
		}
	}
	return -1;
}

PedDevice * pm_get_next_device(const PedDevice *device) {
	PedDevice *dev;

	dev = ped_device_get_next(device);
	if (!dev) {
		pm_debug("Debug: No Next Device Found!\n");
		return NULL;
	}
	if (!ped_device_open(dev))
		return NULL;
	return dev;
}

/* one sector is 512 bytes */

gint pm_pedsector_to_mb(PedSector sec) {
	return (sec * 512) / MEGABYTE_SIZE;
}

PedSector pm_mb_to_pedsector(gint mb) {
	return (mb * MEGABYTE_SIZE) / 512 ;
}

void pm_cleanup() {
	/* FIXME: clear all allocated memory */
}


/* 
	this is broken, silly and stinks

gchar ** pm_get_flag_type_list() {
	PedPartitionFlag part_flag;
	gchar **list;
	gint counter = 0;
	gchar buffer[256];

	for (part_flag = ped_partition_flag_next (0); part_flag;
	     part_flag = ped_partition_flag_next (part_flag)) {

		strncpy(buffer,ped_partition_flag_get_name (part_flag),255);
		list[counter] = malloc(strlen(buffer) + 1);
		counter++;
	}
	if (counter == 0)
		return NULL;
	list[counter] = NULL;
	return list;
}

gchar ** pm_get_disk_type_list() {
	PedDiskType*		disk_type;
	gchar **list;
	gint counter = 0;
	gchar buffer[256];

	for (disk_type = ped_disk_type_get_next (NULL);
	     disk_type; disk_type = ped_disk_type_get_next (disk_type)) {
		strncpy(buffer,disk_type->name,255);
		list[counter] = malloc(strlen(buffer) + 1);
		counter++;
	}
	if (counter == 0)
		return NULL;
	list[counter] = NULL;
	return list;
}

gchar ** pm_get_fs_type_list() {
	PedFileSystemType*	fs_type;
	gchar **list;
	gint counter = 0;
	gchar buffer[256];

	for (fs_type = ped_file_system_type_get_next (NULL);
	     fs_type; fs_type = ped_file_system_type_get_next (fs_type)) {
		strncpy(buffer,fs_type->name,255);
		list[counter] = malloc(strlen(buffer) + 1);
		counter++;
	}
	if (counter == 0)
		return NULL;
	list[counter] = NULL;
	return list;
}
*/








