#ifndef PM_LIBPARTED_H
#define PM_LIBPARTED_H

#include <parted/parted.h>
#include <gtk/gtk.h>

int pm_debug(const char *fmt, ...);

#define MAX_PARTITIONS 128
#define MAX_FILESYSTEMS 128
#define MAX_FLAGS 8

#define MEGABYTE_SIZE (1024*1024)
#define MIN_FREESPACE		(1024 * 2)	/* 1 MB */
#define MEGABYTE (1024 * 1024)
#define MEGABYTE_SECTORS (MEGABYTE / 512)

#define COLOR_EXT2(color) {color.red = 0; color.green = 0; color.blue = 65535;}
#define COLOR_EXT3(color) {color.red = 0; color.green = 0; color.blue = 49152;}
#define COLOR_REIS(color) {color.red = 0; color.green = 0; color.blue = 32768;}
#define COLOR_SWAP(color) {color.red = 0; color.green = 0; color.blue = 16384;}
#define COLOR_XFS(color) {color.red = 0; color.green = 0; color.blue = 32768;}
#define COLOR_JFS(color) {color.red = 0; color.green = 16384; color.blue = 0;}
#define COLOR_16FAT(color) {color.red = 65535; color.green = 0; color.blue = 0;}
#define COLOR_32FAT(color) {color.red = 49152; color.green = 0; color.blue = 0;}
#define COLOR_NTFS(color) {color.red = 32768; color.green = 0; color.blue = 0;}
#define COLOR_RAID(color) {color.red = 0; color.green = 32768; color.blue = 0;}
#define COLOR_LVM(color) {color.red = 0; color.green = 49152; color.blue = 0;}
#define COLOR_FREE(color) {color.red = 65535; color.green = 65535; color.blue = 65535;}

// These could use some cleaning-up... nah, it could use a lot of cleaning up
// might throw these in a struct

PedDevice *devices[64];
PedPartition *partitions[MAX_PARTITIONS];
gint device_count;
gint part_count;
gint fs_count;
gint selected_device;
gint selected_partition;

// are these used?
gint part_primary_count;
gint part_extended_count;
gint part_logical_count;
gint part_freespace_count;
gint part_extended_nr;

// these should be calculated from a device nr
glong selected_device_total_size;
glong selected_device_freespace;

// these should go into a pm_gtk or something
gchar fs_menu_items[64][64];
gchar fs_menu_names[64][64];
gint fs_menu_items_count;
GtkWidget *part_widget;
void display_progress_window();

void pm_set_debug(gboolean deb);

struct fsinfo_struct {
	gchar *name;
	gint color;
	gboolean creatable;
	gboolean resizeable;
	gboolean copyable;
} fsinfo[MAX_FILESYSTEMS];

enum type_enum{
	PRIMARY,
	EXTENDED,
	LOGICAL,
	FREE_SPACE
};

struct partinfo_struct {
	enum type_enum type;
	gchar type_name[64];
	// PedPartition *part; (use ped_disk_get_partition instead)
	PedPartitionFlag flags[MAX_FLAGS];
	gint flag_count;
	PedSector t_start;
	PedSector t_end;
	PedSector start;
	PedSector end;
	PedSector max_size;
	PedSector min_size;

	GtkWidget *part_widget;
	gint row_number;

	struct fsinfo_struct *fs;
	gchar temp_fs_name[64];
	gchar temp_flags_list[128];
	gint num; /* partition number from part table (last number of path) */

	long size; /* size in MB */
	long mb_min_size;
	long mb_max_size;
	gchar partname[32];
	gchar *label;
	gboolean free;
	gboolean unknown;
	gint width;
} partinfo[MAX_PARTITIONS]; // HUGE FIXME! BAAAAD BAAAAD CODING! MALLOC DAMMIT!!! *shrug*

int
pm_solution_check_distant (PedSector req_start, PedSector req_end,
			PedSector soln_start, PedSector soln_end);

gboolean init_arch();

/* 
 * Note: most of this has been directly ported from Parted or QtParted
 * However, functions have been added or adjusted where i felt like it
 */

/*
  changes the label of the device
*/

gboolean pm_do_mklabel (PedDevice **dev, PedDiskType *type);

/*
  makes a filesystem on part
*/

gboolean pm_do_mkfs (PedDevice **dev, int part_nr, 
		  gchar *path, PedFileSystemType *type);

/*
  makes a partition using start and end sectors as guidelines
  doesn't make a filesystem
*/

gboolean pm_do_mkpart (PedDevice **dev,PedPartitionType part_type, 
		    PedSector start, PedSector end);

/*
  makes a partition using start and end sectors as guidelines
  does make a filesystem type fs_type
*/

gboolean pm_do_mkpartfs (PedDevice **dev, PedPartitionType part_type, 
		      PedFileSystemType *fs_type, 
		      PedSector start, PedSector end);
/*
  move the partition part to start-end
*/

gboolean pm_do_move(PedDevice **dev, int part_nr, 
		 gchar *path, 
		 PedSector start, PedSector end);

/*
  change the name of the partition
*/

gboolean pm_do_name(PedDevice **dev,int part_nr, 
		 gchar *name);

/*
  resize the partition part 
  to start-end
*/

gboolean pm_do_resize (PedDevice **dev, int part_nr, 
		    gchar *path, 
		    PedSector start, PedSector end);

gboolean pm_do_resize_part_only (PedDevice **dev, int part_nr, 
		    gchar *path, PedSector start, PedSector end);

/*
  remove the partition num
*/

gboolean pm_do_rm(PedDevice **dev, gint num);

/*
  changes dev to new_dev, returns if succesful
	DISABLED

gboolean select_device(PedDevice **dev, PedDevice **new_dev);
*/

/*
  returns the state of the specified flag
*/

gint pm_get_part_flag_state(PedDevice **dev, int part_nr, 
			 PedPartitionFlag);

/*
  sets the flag to the specified state,
  returns if succesful
*/

gboolean pm_set_part_flag_state(PedDevice **dev, int part_nr, 
			  PedPartitionFlag flag, gint state);

/*
  returns the number of the partition
*/

gint pm_partition_get_minor(PedPartition *part);

/*
  currently not available
*/

gchar * pm_partition_get_fs_name(PedPartition *part);

/*
  returns the length of the partition in MB's
*/

gfloat pm_partition_get_length(PedPartition *part);

/*
  returns how large/small the specified partition can get
*/

gfloat pm_partition_max_resize(PedPartition *part);

gfloat pm_partition_min_resize(PedPartition *part);

/*
  returns if the disk supports extended partitions 
*/

gboolean pm_disk_supports_extended (PedDevice **dev);

/*
  returns if the disk supports partition names
*/

gboolean pm_disk_supports_name (PedDevice **dev);

/*
  returns if the disk has it's max number of primary partitions
  also returns true if it has an extended partition, taking up a primary's slot
*/

gboolean pm_has_max_primary_partitions(PedDevice **device);

/*
  returns if the disk has an extended partition (you can only have one per disk)
*/

gboolean pm_has_extended_partition(PedDevice **device);

/*
  adds all partitions into the partitions-array,
  has a memleak here, should have a cleanup func. FIXME
*/
gboolean pm_scan_partitions(PedDevice **device);

/* clear a certain partinfo */
void initialise_partinfo(gint i);

gboolean pm_update_partinfo();

gint pm_get_arraynumber_from_partinfo_num(gint num);

struct partinfo_struct *pm_get_partition_from_partinfo_num(gint num);

void pm_get_filesystem_list();

void pm_select_device(int device_nr);

gboolean pm_select_update_device(int device_nr);

gboolean grow_over_small_freespace(PedGeometry *geom, PedDisk *disk);

gint pm_get_valid_device(char* device_path);

gboolean partition_warn_busy (PedPartition *part, gchar *path2);

/*
  adds all devices to the devices-array
  has a memleak here, should have a cleanup func. FIXME
*/

void pm_scan_devices();

/*
  returns the next device, NULL if not found
*/

PedDevice *pm_get_next_device(const PedDevice *device);

/* PedSector = 512 bytes */
gint pm_pedsector_to_mb(PedSector sec);

PedSector pm_mb_to_pedsector(gint mb);

void pm_cleanup();

#endif // PM_LIBPARTED_H
