/*
	Copyright (C) 2003 Alex de Landgraaf <alextreme@xs4all.nl>

	wrapper for the pm_do functions,
	should do actions without worrying about fs types etc

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include "pm_libparted.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <parted/parted.h>
#include <parted/debug.h>
#include <parted/linux.h>
#include "pm_wrap.h"
#include "pm_fswrap.h"

void add_fs_item(gchar *item, gchar *name) {
	strcpy(fs_menu_items[fs_menu_items_count],item);
	strcpy(fs_menu_names[fs_menu_items_count],name);
	fs_menu_items_count++;
}

gboolean read_fs_items() {
	fs_menu_items_count = 0;
	add_fs_item("","None");
	add_fs_item("","--");
	if (pm_wrap_mkext3_available()) {
		add_fs_item("ext3","ext3");
	}
	add_fs_item("ext2","ext2");
	add_fs_item("reiserfs","reiserfs");
	if (pm_wrap_mkxfs_available()) {
		add_fs_item("xfs","xfs");
	}
	add_fs_item("","--");
	add_fs_item("linux-swap","Swap");
	add_fs_item("","--");
	add_fs_item("fat16","fat16");
	add_fs_item("fat32","fat32");
	
	if (pm_wrap_mkntfs_available()) {
		add_fs_item("ntfs","ntfs");
	}

	return TRUE;
}

gboolean do_mklabel (PedDevice **dev, PedDiskType *type) {
	return pm_do_mklabel(dev,type);	
}

gboolean do_mkfs (PedDevice **dev, int part_nr ,gchar *path, PedFileSystemType *type) {
	return pm_do_mkfs(dev,part_nr,path,type);
}

gboolean do_mkpart (PedDevice **dev,PedPartitionType part_type, PedSector start, PedSector end) {
	return pm_do_mkpart(dev,part_type,start,end);
}

gboolean do_mkpartfs (PedDevice **dev, PedPartitionType part_type, 
		      char *fs_buffer,
		      int part_nr, 
		      PedSector start, PedSector end) {
	if (strcmp(fs_buffer,"ntfs") == 0) {
		/* resize ntfs filesystem instead */
		return do_mkntfs(dev,part_type,start,end);
	}
	if (strcmp(fs_buffer,"ext3") == 0) {
		return do_mkext3(dev,part_type,start,end);
	}
	if (strcmp(fs_buffer,"xfs") == 0) {
		return do_mkxfs(dev,part_type,start,end);
	}
	return pm_do_mkpartfs(dev,part_type,ped_file_system_type_get(fs_buffer),start,end);
}

gboolean do_move(PedDevice **dev, int part_nr,gchar *path, PedSector start, PedSector end) {
	return pm_do_move(dev,part_nr,path,start,end);
}
gboolean do_name(PedDevice **dev,int part_nr, gchar *name) {
	return pm_do_name(dev,part_nr,name);
}

gboolean check_resize_possible(int part_nr) {
	gint partnr = pm_get_arraynumber_from_partinfo_num(part_nr);
	if (strcmp(partinfo[partnr].temp_fs_name,"ntfs") == 0) {
		return pm_wrap_ntfsresize_available();
	}
	if (partinfo[partnr].max_size == 0 && partinfo[partnr].min_size == 0) {
		return FALSE;
	}
	return TRUE;
}

gboolean do_resize (PedDevice **dev, int part_nr ,gchar *path, PedSector start, PedSector end) {
	gint partnr = pm_get_arraynumber_from_partinfo_num(part_nr);
	if (strcmp(partinfo[partnr].temp_fs_name,"ntfs") == 0) {
		/* resize ntfs filesystem instead */
		return do_ntfsresize(&devices[selected_device],partinfo[partnr].num,(PedSector)partinfo[partnr].end,strdup(partinfo[partnr].partname),(PedSector) partinfo[partnr].start,(PedSector) end);
	}
	else {
		return pm_do_resize(dev,part_nr,path,start,end);
	}
}

gboolean do_rm(PedDevice **dev, gint num) {
	return pm_do_rm(dev,num);
}
