/* 
	Copyright (C) 2004 Alex de Landgraaf <alextreme@xs4all.nl>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
*/

#include <gtk/gtk.h>
#include <parted/parted.h>

#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>

#include "ui.h"
#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "pm_libparted.h"
#include "pm_wrap.h"

/*
  Note: read the parted-doc API first!
  this can be found in /usr/share/doc/parted-doc
  You'll need it to understand certain sections, and pm_lib

  Some guidelines for the source:

  when everything is set up using scan_partitions and scan_devices in 
  pm_lib, the partitions- and devices arrays are filled with the current 
  partitiontable information

  The current selected device can be retrieved with devices[selected_device],
  The current selected partition can be retrieved with partitions[selected_partition]
  
  An Extended partition contains Logical partitions or free space.

  Do use initialise_partinfo to clean out the partinfo array
*/

extern  GtkWidget *MainWindow;
extern  GtkWidget *CreateWindow;
extern  GtkWidget *ResizeWindow;
extern  GtkWidget *ApplyWindow;
extern  GtkWidget *RevertWindow;
extern  GtkWidget *DialogWindow;
extern  GtkWidget *HelpWindow;
extern  GtkWidget *DeleteWindow;
extern  GtkWidget *ConvertWindow;
extern  GtkWidget *MoveWindow;
extern  GtkWidget *TableWindow;
extern  GtkWidget *ErrorWindow;
extern  GtkWidget *ProgressWindow;

enum {
	PART_NUMBER,
	PART_PATH,
	PART_TYPE,
	PART_FS,
	PART_SIZE,
	PART_START,
	PART_END,
	PART_FLAGS,
	N_COLUMNS
};

gint warning_type_global;

void
redirect_stderr_to_pipe(int * saved_fd, int * readable_pipe)
{
  int stderr_fd;
  int pipe_fd[2];
  int oldflags;

  /* Make a copy of the stderr file-descriptor so we can restore it later */
  stderr_fd = fileno(stderr);
  *saved_fd = dup(stderr_fd);

  /* Open up a pipe */
  if (pipe(pipe_fd) == -1) {
    fprintf(stderr, "Could not create pipe; errno=%d!\n", errno);
    exit(-1);
  }
 
  /* Redirect stderr to the writable end of the pipe */
  if ( dup2(pipe_fd[1], stderr_fd) == -1 ) {
    fprintf(stderr, "Could not redirect stderr to the pipe!\n");
    exit(-1);
  }

  /* Since stderr & pipe_fd[1] point to the same thing, close pipe_fd[1] */
  /* NOTE: Do I really need this? */
  close(pipe_fd[1]);

  /* Make sure stderr output is non-blocking (don't hang waiting for EOF) */
  oldflags = fcntl(pipe_fd[0], F_GETFL, 0);
  fcntl(pipe_fd[0], F_SETFL, oldflags|O_NONBLOCK);
  *readable_pipe = pipe_fd[0];
}

void
unredirect_stderr(int saved_fd, int readable_pipe)
{
  /* Close the input (readable) end of the pipe*/
  close(readable_pipe); 

  /* Now undo the effects of redirecting output */ 
  dup2(saved_fd, fileno(stderr));

  /* Now that we no longer need the saved_fd, we can close it */
  close(saved_fd);
}

void
show_error_message(gchar * message)
{
	GtkWidget *error_label = lookup_widget(ErrorWindow,"ErrorLabel");
	int length = strlen(message)-1;
	while (message[length] == '\n')
		length--;
	message[++length] = '\0';
	gtk_label_set_text(GTK_LABEL(error_label), message);
	gtk_widget_show(ErrorWindow);
}

void untoggle_create_types() {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreatePrimaryRadiobutton")),FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateExtendedRadiobutton")),FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateLogicalRadiobutton")),FALSE);
};

void untoggle_create_flags() {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateBootFlagCheckbutton")),FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateHiddenFlagCheckbutton")),FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateRaidFlagCheckbutton")),FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateLVMFlagCheckbutton")),FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(CreateWindow,"CreateLBAFlagCheckbutton")),FALSE);
}

void show_create_fstype() {
	gtk_widget_show(lookup_widget(CreateWindow,"optionmenu4"));
	gtk_widget_show(lookup_widget(CreateWindow,"label27"));	
}

void hide_create_fstype() {
	gtk_widget_hide(lookup_widget(CreateWindow,"optionmenu4"));
	gtk_widget_hide(lookup_widget(CreateWindow,"label27"));	
}

void show_create_flags() {
	gtk_widget_show(lookup_widget(CreateWindow,"CreateBootFlagCheckbutton"));
	gtk_widget_show(lookup_widget(CreateWindow,"CreateHiddenFlagCheckbutton"));
	gtk_widget_show(lookup_widget(CreateWindow,"CreateRaidFlagCheckbutton"));
	gtk_widget_show(lookup_widget(CreateWindow,"CreateLVMFlagCheckbutton"));
	gtk_widget_show(lookup_widget(CreateWindow,"CreateLBAFlagCheckbutton"));
	gtk_widget_show(lookup_widget(CreateWindow,"label65"));
}

void hide_create_flags() {
	gtk_widget_hide(lookup_widget(CreateWindow,"CreateBootFlagCheckbutton"));
	gtk_widget_hide(lookup_widget(CreateWindow,"CreateHiddenFlagCheckbutton"));
	gtk_widget_hide(lookup_widget(CreateWindow,"CreateRaidFlagCheckbutton"));
	gtk_widget_hide(lookup_widget(CreateWindow,"CreateLVMFlagCheckbutton"));
	gtk_widget_hide(lookup_widget(CreateWindow,"CreateLBAFlagCheckbutton"));
	gtk_widget_hide(lookup_widget(CreateWindow,"label65"));
}

void show_busy_cursor( GdkWindow * window )
{
	static GdkCursor * busy_cursor = NULL;

	if (!busy_cursor)
		busy_cursor = gdk_cursor_new(GDK_WATCH);

	/* Make the specified window show the busy cursor */
	gdk_window_set_cursor( window, busy_cursor );

	/* Also make the MainWindow show the busy cursor */
//Fucking global variables, because of MainWindow we get a crash when runing in wizard mode :(
// Maybe should add check if MainWindow==NULL, same for WizardWindow==NULL to work correctly ?
//	gdk_window_set_cursor( MainWindow->window, busy_cursor );
//	gdk_window_set_cursor( WizardWindow->window, busy_cursor );

	while (gtk_events_pending())
		gtk_main_iteration();
}

void show_normal_cursor( GdkWindow * window )
{
	gdk_window_set_cursor( window, NULL );
// FIXME: we can get a crash here if someone will call show_normal_cursor in Wizard mode
	gdk_window_set_cursor( MainWindow->window, NULL );
}

/* List item selection callback */

void tree_selection_changed (GtkTreeSelection *selection, gpointer data) {
	GtkWidget *create_widget = lookup_widget(MainWindow,"CreateButton");
	GtkWidget *resize_widget = lookup_widget(MainWindow,"ResizeButton");
	GtkWidget *delete_widget = lookup_widget(MainWindow,"DeleteButton");
	GtkWidget *move_widget = lookup_widget(MainWindow,"MoveButton");
	GtkWidget *convert_widget = lookup_widget(MainWindow,"ConvertButton");
	/* GtkWidget *refresh_widget = lookup_widget(MainWindow,"RefreshButton"); */

	GtkTreeIter iter;
	GtkTreeModel *model;
	gint num, temp_sel;
	struct partinfo_struct *partition;

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter,PART_NUMBER, &num,-1);
		partition = pm_get_partition_from_partinfo_num(num);
		temp_sel = pm_get_arraynumber_from_partinfo_num(num);
		if (temp_sel < 0 || temp_sel > part_count) {
			pm_debug("Error: selected partition has invalid arraynumber: %d\n",selected_partition);
		}
		else if (partition != NULL && partition->num > -2) {
			selected_partition = temp_sel;
			if (partition->part_widget != NULL) {
				//gtk_button_clicked(GTK_BUTTON(partition->part_widget));
			}
			if (partition->type == FREE_SPACE) {
				if (pm_has_max_primary_partitions(&devices[selected_device]) && (selected_partition < part_extended_nr || part_extended_nr == 0)) {				gtk_widget_set_sensitive(create_widget,FALSE);
				}
				else {
					gtk_widget_set_sensitive(create_widget,TRUE);
				}
				gtk_widget_set_sensitive(resize_widget,FALSE);

				gtk_widget_set_sensitive(delete_widget,FALSE);
				gtk_widget_set_sensitive(move_widget,FALSE);
				gtk_widget_set_sensitive(convert_widget,FALSE);
			}
			else {
				gtk_widget_set_sensitive(create_widget,FALSE);
				if (partition->type == EXTENDED) {
					/*
					  if the partition is extended,
					  extra restraints are necessary.
					  You can't delete the extended partition
					  if logical partitions exist in it.
					  Thankfully for me, you can only have
					  one extended partition...
					*/
					if (part_logical_count > 0) {
						gtk_widget_set_sensitive(resize_widget,FALSE);
						gtk_widget_set_sensitive(delete_widget,FALSE);
						gtk_widget_set_sensitive(move_widget,FALSE);
						gtk_widget_set_sensitive(convert_widget,FALSE);	
					}
					else {
						gtk_widget_set_sensitive(resize_widget,FALSE);
						gtk_widget_set_sensitive(delete_widget,TRUE);
						gtk_widget_set_sensitive(move_widget,FALSE);
						gtk_widget_set_sensitive(convert_widget,FALSE);	
					}
				}
				else {
					if (check_resize_possible(num)) {
						gtk_widget_set_sensitive(resize_widget,TRUE);
					}
					else {
						gtk_widget_set_sensitive(resize_widget,FALSE);					
					}
					gtk_widget_set_sensitive(move_widget,FALSE);
					gtk_widget_set_sensitive(convert_widget,FALSE);	
					gtk_widget_set_sensitive(delete_widget,TRUE);
				}
			}
		}
	}
}

/* Partition button callback */

gboolean click_part_item(GtkWidget *widget, gpointer data) {
	GtkTreePath *path = NULL;
	struct partinfo_struct *partition = (struct partinfo_struct *)data;
	GtkWidget *treeview = lookup_widget(MainWindow,"treeview1");
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	gchar path_string[4];
	sprintf(path_string,"%d",partition->row_number);
	path = gtk_tree_path_new_from_string(path_string);
	if (path != NULL) {
		gtk_tree_selection_select_path(GTK_TREE_SELECTION(selection),
					       (path));
	}
	return TRUE;
}
/*
  updates partinfo-array with all partitions for the current selected_device
  skips the METADATA partition that seems to be before the first partition
  modifies the GtkList of partitions 
*/

void update_partition_widget() {
	GtkWidget *partition_widget = lookup_widget(MainWindow,"hbox4");
	GtkWidget *part_box = gtk_hbox_new(FALSE,5);
	gint i;
	gint win_x;
	gint win_y;
	gint height = 30;
	gint rows = 0;

	if (G_IS_OBJECT(part_widget))
		gtk_container_remove(GTK_CONTAINER(partition_widget),part_widget);
	gtk_window_get_size(GTK_WINDOW(MainWindow),&win_x,&win_y);

	win_x -= 10; // border

	for (i = 0; i < part_count; i++) {
		GtkWidget *part_item = NULL;
		gfloat perc;
		gint width;
		GdkColor color;
		if (partinfo[i].num < 0 && partinfo[i].type != FREE_SPACE) {
			pm_debug("Debug: Skipping partition, partition num invalid\n");
			continue;
		}

		part_item = gtk_button_new();
		perc = (float)partinfo[i].size / selected_device_total_size;
		width = (win_x / 100) * 100 * perc;

		/* Default color */
		color.red = 32768; color.blue = 32768; color.green = 32768;

		if (width <= 20)
			width = 20;
		if (partinfo[i].type == EXTENDED) 
			width = 1;
		if (partinfo[i].type == FREE_SPACE) 
			COLOR_FREE(color)
		if (strcmp(partinfo[i].temp_fs_name,"ext2") == 0)
			COLOR_EXT2(color)
		if (strcmp(partinfo[i].temp_fs_name,"ext3") == 0)
			COLOR_EXT3(color)
		if (strcmp(partinfo[i].temp_fs_name,"reiserfs") == 0)
			COLOR_REIS(color)
		if (strcmp(partinfo[i].temp_fs_name,"xfs") == 0)
			COLOR_XFS(color)
		if (strcmp(partinfo[i].temp_fs_name,"jfs") == 0)
			COLOR_JFS(color)
		if (strcmp(partinfo[i].temp_fs_name,"vfat") == 0)
			COLOR_32FAT(color)
		if (strcmp(partinfo[i].temp_fs_name,"ntfs") == 0)
			COLOR_NTFS(color)
		if (strcmp(partinfo[i].temp_fs_name,"linux-swap") == 0)
			COLOR_SWAP(color)			
		if (partinfo[i].flags[4] == PED_PARTITION_RAID)
			COLOR_RAID(color)
		if (partinfo[i].flags[5] == PED_PARTITION_LVM)
			COLOR_LVM(color)
		pm_debug("Debug: Color: %d, %d, %d\n",color.red,color.green,color.blue);
		if (width >= 100) {
			if (strcmp(partinfo[i].partname,"") != 0)
				gtk_button_set_label(GTK_BUTTON(part_item),partinfo[i].partname);
			else 
				gtk_button_set_label(GTK_BUTTON(part_item),"Free Space");
		}

		pm_debug("Debug: Color: %d, %d, %d\n",color.red,color.green,color.blue);

		gtk_widget_modify_bg(part_item,GTK_STATE_NORMAL,&color);
		gtk_widget_modify_bg(part_item,GTK_STATE_ACTIVE,&color);
		gtk_widget_modify_bg(part_item,GTK_STATE_PRELIGHT,&color);

		gtk_widget_set_size_request(GTK_WIDGET(part_item),width,height);
		partinfo[i].part_widget = part_item;
		partinfo[i].row_number = rows++;
		g_signal_connect(GTK_OBJECT(part_item),"clicked",
				 GTK_SIGNAL_FUNC(click_part_item),(gpointer) &partinfo[i]);
		pm_debug("Debug: Adding partition, number %d:%d width %d type %d name %s size %ld\n",partinfo[i].num, i, width, partinfo[i].type,  partinfo[i].partname, partinfo[i].size);

		gtk_box_pack_start(GTK_BOX(part_box),part_item,TRUE,TRUE,2);
	}
	gtk_container_add(GTK_CONTAINER(partition_widget),part_box);
	part_widget = part_box;
	gtk_widget_show_all(part_box);
	gtk_widget_set_size_request(GTK_WIDGET(MainWindow),win_x,win_y);
}

void set_callback_partition_list() {
	GtkWidget *tree = lookup_widget(MainWindow,"treeview1");
	/* Time to set up the TreeView widget*/

	GtkTreeSelection *select;

	select = gtk_tree_view_get_selection (GTK_TREE_VIEW(tree));
	gtk_tree_selection_set_mode(select,GTK_SELECTION_SINGLE);
	g_signal_connect(G_OBJECT(select),"changed",
			 G_CALLBACK(tree_selection_changed),NULL);
}

gboolean set_create_fs_option_menu() { // make sure read_fs_items() has been run
	GtkWidget *optionmenu = lookup_widget(CreateWindow,"optionmenu4");
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *item;
	int i;

	gtk_option_menu_remove_menu(GTK_OPTION_MENU(optionmenu));
	for (i = 0; i < fs_menu_items_count; i++) {
		if (strcmp(fs_menu_names[i],"--") == 0) {
			item = gtk_separator_menu_item_new();
		}
		else {
			item = gtk_menu_item_new_with_label(fs_menu_names[i]);
		}
		gtk_widget_show(item);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu),item);
	}
	gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu),menu);
	return TRUE;
}


/*
  This function updates the partinfo array, and fills each struct in it with
  current info. Also checks for freespace, and changes the used, free labels
  of the harddisk, updates the progressbar used for showing the percentage
  of free space. Creates a new store for info selected from the array and 
  displays it in the GtkTreeView in MainWindow. 
*/

gboolean update_partition_list() {
	GtkWidget *tree = lookup_widget(MainWindow,"treeview1");
	GtkWidget *used_widget = lookup_widget(MainWindow,"UsedLabel");
	GtkWidget *free_widget = lookup_widget(MainWindow,"FreeLabel");
	GtkWidget *size_bar = lookup_widget (MainWindow,"TotalSizeProgressBar");
	gint i;
	GtkTreeIter iter;
	gchar used_label[64];
	gchar free_label[64];
	gchar percentage[5];
	GtkListStore *store = gtk_list_store_new(N_COLUMNS,
						 G_TYPE_INT,
						 G_TYPE_STRING,
						 G_TYPE_STRING,
						 G_TYPE_STRING,
						 G_TYPE_INT,
						 G_TYPE_INT,
						 G_TYPE_INT,
						 G_TYPE_STRING);

	for (i = 0; i < part_count; i++) {
		gtk_list_store_append(store,&iter);
		gtk_list_store_set(store,&iter,
				   PART_NUMBER,partinfo[i].num,
				   PART_PATH,partinfo[i].partname,
				   PART_TYPE,partinfo[i].type_name,
				   PART_FS,partinfo[i].temp_fs_name,
				   PART_SIZE,(int)(partinfo[i].size),
				   PART_START,(int)partinfo[i].start,
				   PART_END,(int)partinfo[i].end,
				   PART_FLAGS,partinfo[i].temp_flags_list,
				   -1);
	}

	gtk_tree_view_set_model(GTK_TREE_VIEW(tree),GTK_TREE_MODEL(store));
	set_callback_partition_list();	
	sprintf(free_label,_("Free: %ld MB"),selected_device_freespace);
	gtk_label_set_text(GTK_LABEL(free_widget),free_label);
	
	sprintf(used_label,_("Used: %ld MB"),selected_device_total_size - selected_device_freespace);
	gtk_label_set_text(GTK_LABEL(used_widget),used_label);

	/* Progress bar, pretty useless, but it's in there for some reason */
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(size_bar),(selected_device_total_size - selected_device_freespace) / (gfloat) selected_device_total_size);
	sprintf(percentage,_("Used: %d%%"),(gint) (100*((selected_device_total_size - selected_device_freespace) / (gfloat) selected_device_total_size)));
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(size_bar),percentage);
	return TRUE;
}

/*
  executes all changes for when a new device has been selected
  calls update_partition_list to modifiy the partinfo array
*/

void update_interface(gint device) {
	GtkWidget *sizelabel = lookup_widget(MainWindow,"SizeLabel");
	gchar string[256];

	pm_debug("Debug: device %d is: length %d sector %d MB\n",
	       device,(int)devices[device]->length,selected_device_total_size);
	sprintf(string,_("Total size: %ld MB"),selected_device_total_size);
	gtk_label_set_text(GTK_LABEL(sizelabel),string);
}

/*
  Function to completly refresh the partition list
*/

void refresh_devices_and_partitions() {
	GtkWidget *optionmenu = lookup_widget(MainWindow,"MainHarddiskOptionMenu");
	//ped_device_close(devices[selected_device]);

	pm_select_update_device(gtk_option_menu_get_history(GTK_OPTION_MENU(optionmenu)));
	update_interface(selected_device);
	update_partition_list();
	if (part_count < 1) {
		gtk_widget_show(TableWindow);
	}
	else {
		update_partition_widget();

		if (selected_partition < 0) {
			click_part_item(NULL,&partinfo[0]);
		}
		else {
			click_part_item(NULL,&partinfo[selected_partition]);
		}
	}
}
