/* Copyright (C) 2004 Alex de Landgraaf <alextreme@xs4all.nl>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include <glib.h>

#include <support.h>
#include "wizard.h"
#include "ui.h"
#include "pm_libparted.h"

/* defined in header
#define MODE_FREESPACE 1
#define MODE_RESIZE 2
#define MODE_DESTRUCTIVE 3
*/

// I think there are no sense to create partition for /home smaller, than 600
#define MIN_HOME_SIZE 600
#define MIN_SWAP_SIZE 100

/* Default swap size to create is 256 MB, becomes 0 if swap partition is aready created */
gint SWAP_SPACE_SIZE = 256;
// SWAP_SIZE and needed_freespace should use same units
gint needed_freespace;
gint selected_device_nr;


void set_needed_freespace(gint size_mb) {
	if (size_mb > 0) {
		needed_freespace = size_mb;
	}
}

void set_device_nr(gint nr) {
	if (nr > 0) {
		selected_device_nr = nr;
	}
}

/* search for the biggest freespace item with enough freespace (for example needed_freespace)
   if found, return partinfo nr, if not return -1
*/

gint return_part_with_freespace_count( gint needed_space ) {
    int i, free_space_nr = -1;
    for (i = 0; i < part_count; i++) {
	if (partinfo[i].type == FREE_SPACE) {
	    if ( (partinfo[i].size > needed_space) && (free_space_nr = -1) ||
		// if next free space is bigger, than previous
		 (free_space_nr != -1) && (partinfo[i].size > partinfo[free_space_nr].size) ) {
		    free_space_nr = i;
	    }
	}
    }
    return free_space_nr;
}

// Searches for linux swap partitions and returns sum of all swap partitions sizes
// TODO: what to do, if user has several hard disks with swap partitions ?
gint find_swap(){
    int i, swap_size = 0;
    for (i = 0; i < part_count; i++) {
	if (strcmp(partinfo[i].temp_fs_name,"linux-swap") == 0){
	    swap_size = swap_size + partinfo[i].size;
	}
    }
    return swap_size;
}


// Searches for linux partitions and returns biggest linux partition size
// TODO: what to do, if user has several hard disks with linux partitions ?
gint find_linux(){
    int i, size = 0;
    for (i = 0; i < part_count; i++) {
	if ( ( strcmp(partinfo[i].temp_fs_name,"ext2") == 0 ) ||
	     ( strcmp(partinfo[i].temp_fs_name,"xfs") == 0 ) ||
	     ( strcmp(partinfo[i].temp_fs_name,"reiserfs") == 0 ) ||
	     ( strcmp(partinfo[i].temp_fs_name,"ext3") == 0 ) ) {
	    if ( partinfo[i].size > size ) {
		size = partinfo[i].size;
	    }
	}
    }
    return size;
} 

//Returns true if there already are enough size linux and swap partitionions
gboolean can_use_existing() {

    return (find_linux() >= needed_freespace) && (find_swap() >= MIN_SWAP_SIZE);

}
/*
	returns the advised partition type:
	primary if there are three partitions left
	logical if primary doesn't have enough, and extended exists
	extended if it doesn't yet exist
	-1 otherwise: abort and show an error message (yeah, as if)
*/

gint pm_return_available_parttype() {
	if (pm_get_primary_partitions_count(&devices[selected_device]) + 3 <= 4) {
		return 0; /* PED_PARTITION_PRIMARY; */
	}
	if (pm_has_extended_partition(&devices[selected_device])) {
		return PED_PARTITION_LOGICAL;
	}
	else return PED_PARTITION_EXTENDED;
}

gboolean ntfs_part_exists() {

}

gboolean resize_to_freespace() {

}

/*
	uses the first available freespace that is large enough,
	add a root partititon, formatted as ext3, that has max_size - 256MiB
	and if swap partition isn't already available on selected disk then
	add a swap partititon, formatted as swap, that has the remaining space
	return FALSE if any of the operations failed
*/ 

gboolean apply_freespace() {
    pm_select_update_device(selected_device);
// TODO: let user to choose if they want to create a partition for other OS ;)
// Ideally it should detect if there are any primary FAT or NTFS partition
//	needed_freespace = needed_freespace + 4096
/* This should be done in morphixinstaller:
  Linux partitions should be detected and if there are /home partition there should
  be suggestion to leave /home as is and format only root partition
  also morphixinstaller should automatically add users, which are in /home partition */
// If selected hard disk alredy contains sufficient size swap partition, then we need a less free space
    if ( find_swap() >= MIN_SWAP_SIZE ){
        needed_freespace = needed_freespace - SWAP_SPACE_SIZE;
        SWAP_SPACE_SIZE = 0;
    // else if find_swap > 0 
    // handle situation when there are small swap space and not enough free space for root+swap
    // but enough free space for root
    }
    selected_partition = return_part_with_freespace_count(needed_freespace);
// TODO: Handle sitations, when there are several available free spaces, enought size for root,
// but not enought continuos free space for root+swap
    if (pm_return_available_parttype() == PED_PARTITION_EXTENDED) {
    	if (!pm_do_mkpart(&devices[selected_device],PED_PARTITION_EXTENDED,partinfo[selected_partition].start,partinfo[selected_partition].end)) {
		pm_debug("Error: couldn't partition device: %d, partition %d as EXTENDED during MODE_FREESPACE (I couldn't allocate an extended partition large enough for the necessary logical partitions)\n",selected_device,selected_partition);
		return FALSE;
	}
	pm_select_update_device(selected_device);
    }
    selected_partition = return_part_with_freespace_count(needed_freespace);
// If freespace is bigger enought, than is needed for root, then create root and home partitions
// TODO: Handle sitations, when there are several available free spaces, for 
// example one is enought size for root + swap and second is enought size for /home
    if (partinfo[selected_partition].size >= needed_freespace + MIN_HOME_SIZE){
	// if there are lots of free space, then we can make root partition bigger ;)
	if ( partinfo[selected_partition].size > needed_freespace * 2 + MIN_HOME_SIZE * 4 ){
	    needed_freespace = needed_freespace * 2;
	}
	long long home_end = partinfo[selected_partition].end - needed_freespace * 1024 * 2; /* MiB */
	if ( !do_mkpartfs(&devices[selected_device],pm_return_available_parttype(), "ext3",
		selected_partition, partinfo[selected_partition].start,home_end) ) {
		pm_debug("Error: couldn't partition device: %d, partition: %d, end partition pedsector: %d fs: ext3 type: %d\n", 
			    selected_device, selected_partition, home_end, pm_return_available_parttype() );
		return FALSE;
	}
    }
    pm_select_update_device(selected_device);

    selected_partition = return_part_with_freespace_count( needed_freespace );

    long long root_end = partinfo[selected_partition].end - SWAP_SPACE_SIZE * 1024 * 2; /* 256MiB */
// AFAIK do_mkpartfs uses *gchar for fstype, not ped_file_system_type_get("ext3")
    if (!do_mkpartfs(&devices[selected_device],pm_return_available_parttype(), "ext3",
	selected_partition, partinfo[selected_partition].start,root_end)) {
	pm_debug("Error: couldn't partition device: %d, partition: %d, end partition pedsector: %d fs: ext3 type: %d\n", 
		    selected_device, selected_partition, root_end, pm_return_available_parttype() );
	return FALSE;
    }
    pm_select_update_device(selected_device);
//	selected_partition++; /* is this correct? is the partition after a freshly-made primary/logical partition always the remaining freespace? */
// I think more correct would be call return_part_with_freespace_count( SWAP_SPACE_SIZE )
// But we shouldn't forger, that available_parttype could change

//	If sufficient size swap partition is already available on selected disk
    if (SWAP_SPACE_SIZE > 0) {
// It seems, that after making root partition available 
// free space for swap partition is little smaller, than 256 Mb
	gint swap = SWAP_SPACE_SIZE - 2;
	selected_partition = return_part_with_freespace_count( swap );
	if (!do_mkpartfs(&devices[selected_device],pm_return_available_parttype(), "linux-swap",
	    selected_partition, partinfo[selected_partition].start,partinfo[selected_partition].end)) {
	    pm_debug("Error: couldn't partition device: %d, partition: %d, end partition pedsector: %d fs: swap type: %d\n", 
		    selected_device, selected_partition, partinfo[selected_partition].end, pm_return_available_parttype() );
	    return FALSE;
	}
    }
    pm_select_update_device(selected_device);
    return TRUE;
}

/*
	try to resize any partitions available in order to get the minimal
	freespace amount. as ntfs partitions are more difficult to partition,
	pop up a warning asking if the user is sure he defragmented his partition.
	If the amount of free space already is available, use that amount.
	If after resizing the amount of free space still isn't enough, 
	return FALSE.
*/

gboolean apply_resize() {
	if (return_part_with_freespace_count(needed_freespace)) {
		return apply_freespace();
	}
	if (ntfs_part_exists()) {
//		show_warning_message("You seem to have NTFS partitions on your harddisk. Be aware that you must have defragmented your NTFS partitions before attempting to resize them outside of Windows(tm).\n\nFailure to do so will lead to data corruption. If you have defragmented your partitions make sure you have a backup of all data on these partitions, as this remains an experimental procedure\n",WARNING_MESSAGE_RESIZE);
	}
	return resize_to_freespace();
}

/* 
	Remove all existing partitions
	make 256mb swap partition
	use all remaining free space for ext3 partition
*/

gboolean apply_clean() {
// Recreating disk label (partition table) should work with all hard disks - new and old ;)
	do_mklabel(&devices[selected_device],ped_disk_type_get("msdos"));
//Now we have disk with clean partition table and can partition this freespace ;)
	return apply_freespace();
}

gboolean wizard_apply_mode(gint mode_nr) {

	if (mode_nr == MODE_FREESPACE)
		return apply_freespace();
	if (mode_nr == MODE_RESIZE)
		return apply_resize();
	if (mode_nr == MODE_DESTRUCTIVE)
		return apply_clean();
	pm_debug("Error: Invalid mode\n");
	return FALSE;
}

gboolean check_enough_freespace(gint size_mb) {
	if (return_part_with_freespace_count(needed_freespace) >= 0) 
		return TRUE;
	return FALSE;
}

gboolean resizeable_partitions(int device_nr,int size) {
	/* FIXME: check if enough partition-space can be extracted
           only enable this when there isn't enough free space too though... */
	return FALSE;
}

gboolean return_mode_available(gint mode_nr) {
	if (needed_freespace <= 0 || selected_device_nr < 0) {
		// invalid freespace set or invalid device nr
		// might blurt some debug stuff in here
		// or pop up an error dialog
		return FALSE;
	}
	if (mode_nr == MODE_FREESPACE) {
		if (check_enough_freespace(needed_freespace)) {
			return TRUE;
		}
		else {
			return FALSE;
		}
	}
	if (mode_nr == MODE_RESIZE) {
		if (resizeable_partitions(selected_device_nr,needed_freespace)) {
			return TRUE;
		}
		else {
			return FALSE;
		}
	}
	if (mode_nr == MODE_DESTRUCTIVE) {
		return TRUE;
	}
	return FALSE;
}

